-------------
-- DRAWING --
-------------

--[[
	All the drawing functions (so far):
	
	minigame.draw(imagedata, x, y)
		> Draws an image on the screen.
			imagedata: An imagedata object or a table
			x: X coordinate
			y: Y coordinate
	
	minigame.point(x, y)
		> Draws a point on the screen.
			x: X coordinate
			y: Y coordinate
	
	minigame.rectangle(mode, x, y, width, height)
		> Draws a rectangle (filled or outlined) on the screen.
			mode: "fill" or "line"
			x: X coordinate
			y: Y coordinate
			width: Rectangle's width
			height: Rectangle's height
	
	minigame.circle(mode, x, y, radius)
		> Draws a circle (filled or outlined) on the screen.
			mode: "fill" or "line"
			x: X coordinate
			y: Y coordinate
			radius: Circle's radius
	
	minigame.polygon(mode, x1, y1, x2, y2, x3, y3...)
		> Draws a polygon (filled or outlined) on the screen.
		> Takes as many arguments as there are vertices (excluding the draw mode).
		> Number of arguments (excluding the draw mode) must be at least 6, and a multiple of 2.
			mode: "fill" or "line"
			x1, x2, x3...: The N'th point's X coordinate
			y1, y2, y3...: The N'th point's Y coordinate
	
	minigame.line(x1, y1, x2, y2)
		> Draws a line between two points on the screen.
			x1: The first point's X coordinate
			y1: The first point's Y coordinate
			x2: The second point's X coordinate
			y2: The second point's Y coordinate
	
	minigame.print(string, x, y[, size])
		> Prints a string on the screen.
			string: Your text
			x: X coordinate
			y: Y coordinate
			size (optional): The scale of your text. Defaults to 1
	
	minigame.printSize(string[, size])
		> Returns the size (in pixels) your text will take
			string: Your text
			size (optional): The scale of your text. Defaults to 1
			
			Returns: width, height
			width: The amount of pixels (horizontally) your text takes
			height: The amount of pixels (vertically) your text takes
	
	minigame.printWidth(string[, size])
		> Returns the size (in pixels) your text will take
			string: Your text
			size (optional): The scale of your text. Defaults to 1
			
			Returns: width
			width: The amount of pixels (horizontally) your text takes
	
	minigame.printHeight(string[, size])
		> Returns the size (in pixels) your text will take
			string: Your text
			size (optional): The scale of your text. Defaults to 1
			
			Returns: height
			height: The amount of pixels (vertically) your text takes
	
	minigame.number(number, x, y[, bigmode, size])
		> Prints a number on the screen (with digital number font).
			number: Your number
			x: X coordinate
			y: Y coordinate
			bigmode (optional): If true, uses larger digital number font. Defaults to false
			size (optional): The scale of your number. Defaults to 1
	
	minigame.numberSize(number[, bigmode, size])
		> Returns the size (in pixels) your number will take
			number: Your number
			bigmode (optional): If true, uses larger digital number font. Defaults to false
			size (optional): The scale of your number. Defaults to 1
			
			Returns: width, height
			width: The amount of pixels (horizontally) your number takes
			height: The amount of pixels (vertically) your number takes
	
	minigame.numberWidth(number[, bigmode, size])
		> Returns the size (in pixels) your number will take
			number: Your number
			bigmode (optional): If true, uses larger digital number font. Defaults to false
			size (optional): The scale of your number. Defaults to 1
			
			Returns: width
			width: The amount of pixels (horizontally) your number takes
	
	minigame.numberHeight(number[, bigmode, size])
		> Returns the size (in pixels) your number will take
			number: Your number
			bigmode (optional): If true, uses larger digital number font. Defaults to false
			size (optional): The scale of your number. Defaults to 1
			
			Returns: height
			height: The amount of pixels (vertically) your number takes
	
	minigame.setColor(c)
		> Sets the color going to be used to draw objects after this call
			c: The color value (0, 0.2, 0.4, 0.6, 0.8 or 1). 0 being 100% transparent and 1 being 100% opaque
	
	minigame.getColor()
		> Returns the last color set to be used to draw objects
			Returns: c
			c: The color value (0, 0.2, 0.4, 0.6, 0.8 or 1). 0 being 100% transparent and 1 being 100% opaque
	
	minigame.setBackground(c)
		> Sets the color of the game's background.
			c: The color value (0, 0.2, 0.4, 0.6, 0.8 or 1). 0 being 100% transparent and 1 being 100% opaque
	
	minigame.pixelMode(mode)
		> Sets whether transparent-colored pixels should replace the pixels drawn below them or mix with them.
			mode: "replace" or "blend"
	
	minigame.setScissor(x, y, width, height)
		> Sets a scissor that limits where pixels can be drawn in a rectangular area.
		> When used without arguments, it disables the previous scissor.
			x: X coordinate
			y: Y coordinate
			width: The scissor's width
			height: The scissor's height
	
	minigame.translate(x, y)
		> Sets a new origin point for drawn object's coordinates.
		> When used without arguments, it disables the previous translation.
			x: X coordinate
			y: Y coordinate
	
	minigame.origin()
		> Sets the origin point back to default.
	
	minigame.setLineWidth(width)
		> Defines the width of the line used for outline drawables, such as minigame.line or geometric shapes on "line" mode
			width: The new line width
	
	minigame.setPointSize(size)
		> Defines the radius of the point for minigame.point
			size: The new point radius
]]



--------------
-- MINIGAME --
--------------

--[[
	All the set-related functions (so far):
	
	minigame.setLives(lives)
		> Sets the amount of lives the player has during the game.
			lives: New amount of lives player will have. Must be a number or false (disables lives)
	
	minigame.addLives(number)
		> Increments the amount of lives the player has during the game.
			number: The amount of lives to add
	
	minigame.removeLives(number)
		> Decrements the amount of lives the player has during the game.
			number: The amount of lives to remove
	
	minigame.getLives()
		> Returns the amount of lives the player has during the game.
		
		Returns: number
			number: The amount of lives
	
	minigame.gameOver()
		> Forces the game to stop and returns to menu.
	
	minigame.setScore(number)
		> Sets the score the player has during the game
			number: New score the player will have
	
	minigame.addScore(number)
		> Increments the score the player has during the game
			number: The amount of points to add
	
	minigame.removeScore(number)
		> Decrements the score the player has during the game
			number: The amount of points to remove
	
	minigame.getScore()
		> Returns the score the player has during the game.
		
		Returns: number
			number: The amount of points
	
	minigame.getHighscore()
		> Returns the highscore the player reached so far.
		
		Returns: number
			number: The player's highscore
	
	minigame.resetLevel()
		> Reloads the game from the last level.
	
	minigame.levelUp()
		> Reloads the game from the next level. If no level is available, runs [game]_gameOver().
	
	minigame.buttonDown(button)
		> Returns whether or not a button is being held by the player.
			button: The button to be checked
		
		Returns: boolean
			boolean: True if button is being held, false otherwise
	
	minigame.lastButton([button])
		> Returns the last button the player pressed. If button is specified, returns boolean if button matches the last one pressed.
			button (optional): The button to be checked
			
		Returns: button or boolean
			button: The last button pressed
			boolean: True if specified button corresponds to last button pressed. False otherwise
	
	minigame.isPaused()
		> Returns whether or not the game is paused
		
		Returns: boolean
			boolean: True if game is paused, false otherwise
	
	minigame.setTime(time)
		> Sets a new time limit for the game.
			time: New time limit. Must be a number or false (disables timer)
	
	minigame.getTime([mode])
		> Returns the time left on the timer.
			mode (optional): "string" or "number". Defaults to "string"
		
		Returns: time
			time: The amount of time left. If mode is "string", returns formatted timestamp. Otherwise, returns number (in seconds)
	
	minigame.addTime(number)
		> Increments the time left on the timer.
			number: The amount of seconds to add
	
	minigame.removeTime(number)
		> Decrements the time left on the timer.
			number: The amount of seconds to remove
	
	minigame.setSpeed(number)
		> Sets the new speed at which the game runs.
			number: The new game's speed. If smaller than 1, multiples of 0.2. Otherwise, multiples of 1
	
	minigame.getSpeed()
		> Returns the speed at which the game runs.
		
		Returns: number
			number: The speed at which the game is running
	
	minigame.addSpeed(number)
		> Increments the speed at which the game runs.
			number: The amount to add to the game's speed
	
	minigame.removeSpeed(number)
		> Decrements the speed at which the game runs.
			number: The amount to remove to the game's speed
	
	minigame.setProgress(number)
		> Sets the amount of progress made in the game (for visual representation)
			number: The amount of progress made in the game or in a level (from 0 to 1)
	
	minigame.getProgress()
		> Returns the amount of progress made in the game
		
		Returns: number
			number: The amount of progress made in the game or in a level (from 0 to 1)
]]



---------------
-- UTILITIES --
---------------

--[[
	All the data-managing new functions (so far):
	
	string:split(string, delimiter)
		> Divides a string into a table of segments, based on a given delimiter.
			string: The string to be split
			delimiter: The character that defines where the splitting takes place
	
	distance(x1, y1, x2, y2)
		> Returns the exact distance between two points.
			x1: The first point's X coordinate
			y1: The first point's Y coordinate
			x2: The second point's X coordinate
			y2: The second point's Y coordinate
		
		Returns: number
			number: The distance between the two points
	
	table.contains(table, entry)
		> Returns whether or not a table contains a specific item and, if so, return its index position.
			table: The table to be analized
			entry: The item to be found within that table
		
		Returns: index or false
			false: Returned when the table does not contain the specified item
			index: The position within the table at which the item was first found
	
	table.copy(table)
		> Returns an independent, identical copy of a table
			table: The base table to be copied
		
		Returns: table
			table: The copy of the given table
	
	round(number[, decimal])
		> Rounds down a number up to certain amounts of decimals
			number: The number to be rounded down
			decimal (optional): The amount of decimal digits to be rounded. Defaults to 2
		
		Returns: number
			number: The rounded down number
	
	addZeros(number[, lenght])
		> Adds the character "0" to the start of a number until it matches the desired string's lenght.
			number: The number to be formatted
			lenght (optional): The size of the string that the number must match. Defaults to 2
		
		Returns: string
			string: The formatted string with the number with its "0"s added
	
	inside(x1, y1, w1, h1, x2, y2, w2, h2)
		> Detects when a rectangle is contained within another rectangle.
			x1, y1, w1, h1: The inner rectangle's X coordinate, Y coordinate, width and height
			x2, y2, w2, h2: The outer rectangle's X coordinate, Y coordinate, width and height
			
		Returns: boolean
			boolean: Whether or not the inner rectangle is contained within the outer rectangle
	
	detectCollision(px, py, v1x, v1y, v2x, v2y, v3x, v3y)
		> Returns whether or not a point is contained within a triangle.
			px: The point's X coordinate
			py: The point's Y coordinate
			v1x: The triangle's first vertex's X coordinate
			v1y: The triangle's first vertex's Y coordinate
			v2x: The triangle's second vertex's X coordinate
			v2y: The triangle's second vertex's Y coordinate
			v3x: The triangle's third vertex's X coordinate
			v3y: The triangle's third vertex's Y coordinate
		
		Returns: boolean
			boolean: Whether or not a point is within a triangle
]]



--------------------
-- GAME FUNCTIONS --
--------------------

--[[
	All the game functions (so far):
	
	[game].name
		> Variable that holds the game's name. Must be a string.
		> If not present, the game's filename will be used instead.
	
	[game].author
		> Variable that holds the game author's name. Must be a string.
		> If not present, defaults to "unknown".
	
	[game].icon
		> Variable that holds the game's menu icon. Must be a string (with filename), an ImageData or a pixel table.
		> If not present but file "[game]_icon.png", icon is loaded. Otherwise, defaults to screenshot of level 1.
		> If the icon's width is larger than 16px, the icon is animated.
	
	[game].arcadeMode
		> Variable that defines whether or not the player can play levels not reached before. Must be a boolean.
		> If not present, defaults to false.
	
	[game].maxLevel
		> Variable that defines the maximum level the player can reach in the game.
		> If not present, a stress test will run to determine it up to 100.
	
	[game]_load(level)
		> Called whenever the game is loaded (or reloaded)
			level: The current level the player is at
	
	[game]_update(dt)
		> Called between each frame of the game's run
			dt: Delta time between each frame
	
	[game]_timeUp()
		> Called whenever the game's timer reaches 0.
		> If not present, minigame.timeUp() will be called, resulting in a level reload and a life removed.
	
	[game]_draw()
		> Called in each frame, draws any objects called within it.
	
	[game]_buttonPressed(button)
		> Called whenever the player presses a button.
			button: The last button the player pressed
	
	[game]_buttonReleased(button)
		> Called whenever the player releases a button.
			button: The last button the player released
	
	[game]_gameOver()
		> Called whenever the game fails to load a next level, timer runs out or lives reach 0.
		> If not present, minigame.gameOver will be called.
]]