---------------
-- FILE TREE --
---------------

--[[
	FOLDER [set name]
		FOLDER buttons
			FILE big.png
			FILE big_down.png
			FILE up.png
			FILE up_down.png
			FILE left.png
			FILE left_down.png
			FILE down.png
			FILE down_down.png
			FILE right.png
			FILE right_down.png
			FILE settings.png
			FILE settings_down.png
			FILE pause.png
			FILE pause_down.png
			FILE [custom button name].png
			FILE [custom button name]_down.png
		FOLDER graphics
			FILE minigame.png
			FILE minigameoverlay.png
			FILE minigamebackground.png
			FILE font.png
			FILE smallnumbers.png
			FILE bignumbers.png
		FOLDER screens
			FILE main.png
			FILE main_on.png
			FILE [custom screen name].png
			FILE [custom screen name]_on.png
			FILE [custom screen name].lua
		FILE buttons.txt
		FILE screens.txt
]]



--------------------
-- CUSTOM BUTTONS --
--------------------

--[[
	Default buttons: "big", "up", "left", "down", "right", "pause" and "settings"
	
	FILE [custom button name].png
		> The image to be rendered on-screen of the button when not being pressed.
	
	FILE [custom button name].png
		> The image to be rendered on-screen of the button when being held down.
	
	FILE buttons.txt
		> Loads custom settings for default and custom buttons.
		> Each button uses one line of text for its settings.
		> If default buttons settings aren't changed, defaults to coincide with "default" set.
		
		[button name] = [shape], [x], [y], [width/radius], [height], [edge radius], [keyboard keys]
		
		button name: The name of the default or custom button. Must coincide with graphics file name
		shape: The geometric shape of the button.
			"circle": uses [x], [y] and [radius] to define its area of use
			"rectangle": uses [x], [y], [width] and [height] to define its area of use
			"ellipse": uses [x], [y], [width] and [height] to define its area of use
			"arrowup": uses [x], [y], [width] and [height] to define its area of use
			"arrowleft": uses [x], [y], [width] and [height] to define its area of use
			"arrowdown": uses [x], [y], [width] and [height] to define its area of use
			"arrowright": uses [x], [y], [width] and [height] to define its area of use
			"roundrectangle": uses [x], [y], [width], [height] and [edge radius] to define its area of use
		x: X coordinate
		y: Y coordinate
		radius: The radius of the shape. Exclusive for shape "circle"
		width: The width of the shape
		height: The height of the shape
		edge radius: The radius of the curved edges of the shape. Exclusive for shape "roundrectangle"
]]



--------------------
-- CUSTOM SCREENS --
--------------------

--[[
	Default screen: "main"
	
	FILE [custom screen name].png
		> The image to be rendered on-screen of the custom screen when not displaying images.
	
	FILE [custom screen name]_on.png
		> The image to be rendered on-screen of the custom screen when displaying images.
	
	FILE [custom screen name].lua
		> The file containing the code that is loaded whenever the custom screen is rendered. Used to display images on the custom screen.
		> Coding is identical to game graphics coding, and uses the same functions, such as presented in the 'minigame' class.
		> Notice: this file does not work for the main screen.
	
	FILE screens.txt
		> Loads custom settings for default and custom screens.
		> Each screen uses one line of text for its settings.
		> If the screen settings aren't changed, defaults to coincide with "default" set.
		
		[screen name] = [x], [y]
		screen name: The name of the main or custom screen. Must coincide with graphics file name
		x: X coordinate
		y: Y coordinate
]]



---------------------
-- CUSTOM GRAPHICS --
---------------------

--[[
	FILE minigame.png
		> The image to be rendered on-screen as the minigames set. The size of this image defines the size of the game window.
		> This image is artificially colored depending on the minigame set color in settings.
	
	FILE minigameoverlay.png
		> An image to be rendered on top of the minigame set. Optional.
	
	FILE minigamebackground.png
		> An image to be rendered below the minigame set. Optional.
	
	FILE font.png
		> The images that represent the characters to be drawn in text messages. If not present, defaults to font in "default" set.
		> All letters must be of equal width.
		> Glyphs: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789.!?:-#&()[]{}<>"
	
	FILE smallnumbers.png
		> The images that represent the characters to be drawn in numeric displays. If not present, defaults to font in "default" set.
		> All digits must be of equal width.
		> Glyphs: "0123456789"
	
	FILE bignumbers.png
		> The images that represent the characters to be drawn in large numeric displays. If not present, defaults to font in "default" set.
		> All digits must be of equal width.
		> Glyphs: "0123456789"
]]