--Drawing defaults
minigame.lineWidth = 1
minigame.pointSize = 1
minigame.color = 1
minigame.background = 0
minigame.blendmode = "blend"

--Translation defaults
minigame.x = 1
minigame.y = 1

--Scissor defaults
minigame.scissorX = 1
minigame.scissorY = 1
minigame.scissorW = pixelsH
minigame.scissorH = pixelsV

minigame.width = pixelsH
minigame.height = pixelsV

--[[
	Progress map:
	
	■ - Documented
	□ - Not documented/documentation unnecessary
	
	√ - Fully implemented
	≈ - Partially implemented or in need of optimization
	≠ - Not implemented/implemented in a rough way
]]

-- ■ √
function minigame.pixelMode(s)
	if type(s) == "string" then
		s = string.lower(s)
	end
	if s == "blend" or s == "b" then
		minigame.blendmode = "blend"
	elseif s == "replace" or s == "r" then
		minigame.blendmode = "replace"
	end
end

-- □ √
function minigame.fixedColor(c)
	if type(c) ~= "number" then
		c = 1
	end
	if c > 1 then
		c = 1
	elseif c < 0 then
		c = 0
	end
	
	return math.floor(c*10/2)*2/10
end

-- ■ √
function minigame.draw(imgdata, x, y)
	local t = imageToData(imgdata)
	
	if pixelmode then --Screen other than main
		for x = 1, #t do
			for py = 1, #t[px] do
				local ax, ay = px-1, py-1
				if t[px][py] and minigame.getColor() == pixelmode then
					love.graphics.rectangle("fill", px, py, 1, 1)
				end
			end
		end
	end
	
	local nx, ny = minigame.x+x-1, minigame.y+y-1
	
	for px = 1, #t do
		for py = 1, #t[px] do
			local ax, ay = nx+px-1, ny+py-1
			if minigame.onScissor(ax, ay) then
				local p = t[px][py]
				if p == false then p = 0 end
				setPixel(ax, ay, math.max(0, p-(1-minigame.color)))
			end
		end
	end
end

-- ■ √
function minigame.rectangle(mode, x, y, width, height)
	if pixelmode then --Screen other than main
		if minigame.getColor() == pixelmode then
			if mode == "fill" then
				love.graphics.rectangle(mode, x, y, width, height)
			elseif mode == "line" then
				local l = math.ceil(minigame.lineWidth/2)
				local s = math.floor(minigame.lineWidth/2)
				love.graphics.rectangle("fill", x+l, y-s, width-l*2, minigame.lineWidth) -- ^
				love.graphics.rectangle("fill", x-s, y-s, minigame.lineWidth, height+s*2) -- <
				love.graphics.rectangle("fill", x+l, y+height-l, width-l*2, minigame.lineWidth) -- v
				love.graphics.rectangle("fill", x+width-l, y-s, minigame.lineWidth, height+s*2) -- >
			end
		end
		return
	end
	
	local nx, ny = minigame.x, minigame.y
	if mode == "fill" then
		for px = x, x+width-1 do
			for py = y, y+height-1 do
				local ax, ay = nx+px-1, ny+py-1
				if minigame.onScissor(ax, ay) then
					setPixel(ax, ay, minigame.color)
				end
			end
		end
	elseif mode == "line" then
		local stx, sty, sts = x, y, minigame.lineWidth
		if sts ~= 1 and math.mod(sts, 2) ~= 0 then
			stx = stx-math.floor((sts-1)/2)
			sty = sty-math.floor((sts-1)/2)
		elseif sts ~= 1 then
			stx = stx-math.floor(sts/2)
			sty = sty-math.floor(sts/2)
		end
		minigame.rectangle("fill", stx, sty, width, sts)
		minigame.rectangle("fill", stx, sty+1, sts, height-2*sts)
		minigame.rectangle("fill", stx+width-1, sty+1, sts, height-2*sts)
		minigame.rectangle("fill", stx, sty+height-1, width, sts)
	end
end

-- ■ √
function minigame.circle(mode, x, y, radius, d)
	if pixelmode then --Screen other than main
		if minigame.getColor() == pixelmode then
			love.graphics.circle(mode, x, y, radius, d or 32)
		end
	end
	
	for xx = x-radius-math.ceil(minigame.lineWidth/2), x+radius+math.ceil(minigame.lineWidth/2) do
		for yy = y-radius-math.ceil(minigame.lineWidth/2), y+radius+math.ceil(minigame.lineWidth/2) do
			local nx, ny = minigame.x+xx, minigame.y+yy
			if minigame.onScissor(nx, ny) and distance(xx, yy, x, y) <= radius+minigame.lineWidth/2 then
				if mode == "line" then
					if distance(xx, yy, x, y) >= radius-minigame.lineWidth/2 then
						setPixel(nx, ny, minigame.color)
					end
				elseif distance(xx, yy, x, y) <= radius then
					setPixel(nx, ny, minigame.color)
				end
			end
		end
	end
end

-- ■ √
function minigame.line(x1, y1, x2, y2)
	local points = {}
	local sub_points = {}
	
	local sx = math.min(x1, x2)
	local sy = y1
	local ww = math.abs(x1-x2)
	local hh = y2-y1
	if sx == x2 then
		sy = y2
		hh = y1-y2
	end
	
	local d = math.ceil(math.max(math.abs(x1-x2), math.abs(y1-y2)) + .01)
	
	for i = 0, d do
		local f = i/d
		local px = math.floor(sx+f*ww)
		local py = math.floor(sy+f*hh)
		table.insert(points, px)
		table.insert(points, py)
	end
	
	for i = 1, #points-1, 2 do
		if table.contains(sub_points, {points[i], points[i+1]}) then
			--Do not draw the same point twice, regardless of pixelmode
		elseif not pixelmode then
			local nx, ny = math.ceil(points[i])+minigame.x-1, math.ceil(points[i+1])+minigame.y-1
			if minigame.onScissor(nx, ny) then
				if minigame.lineWidth > 1 then
					minigame.circle("fill", nx, ny)
				else
					setPixel(nx, ny, minigame.color)
				end
			end
		elseif pixelmode == minigame.getColor() then
			love.graphics.rectangle("fill", points[i], points[i+1], 1, 1)
		end
		table.insert(sub_points, {points[i], points[i+1]})
	end
end

-- ■ √
function minigame.polygon( ... )
	local args = { ... }
	local mode = args[1]
	table.remove(args, 1)
	
	local outter = {unpack(args)}
	table.insert(outter, args[1])
	table.insert(outter, args[2])
	if #args == 6 then
		args = {args} -- Already a triangle
	else
		args = love.math.triangulate(unpack(args))
	end
	
	if pixelmode then --Screen other than main
		if pixelmode == minigame.getColor() then
			if mode == "fill" then
				for i, v in ipairs(args) do
					love.graphics.polygon(mode, unpack(v))
				end
			elseif mode == "line" then
				for i = 3, #outter-1, 2 do
					love.graphics.line(outter[i-2], outter[i-1], outter[i], outter[i+1])
				end
			end
		end
	end
	
	if not pixelmode then
		if mode == "fill" then
			local minx, miny, maxx, maxy
			for i, v in ipairs(args) do
				for j = 1, #v-1, 2 do
					if not minx or v[j] < minx then
						minx = v[j]
					end
					if not miny or v[j+1] < miny then
						miny = v[j+1]
					end
					if not maxx or v[j] > maxx then
						maxx = v[j]
					end
					if not maxy or v[j+1] > maxy then
						maxy = v[j+1]
					end
				end
			end
			
			local filled = {}
			for i, v in ipairs(args) do
				for x = minx, maxx do
					for y = miny, maxy do
						local ax, ay = x+minigame.x-1, y+minigame.y-1
						if minigame.onScissor(ax, ay) then
							if detectCollision(x, y, unpack(v)) and not table.contains(filled, {x, y}) then
								setPixel(ax, ay, minigame.color)
								table.insert(filled, {x, y})
							end
						end
					end
				end
			end
		else
			for i = 3, #outter-1, 2 do
				minigame.line(outter[i-2], outter[i-1], outter[i], outter[i+1])
			end
		end
	end
end

-- ■ √
function minigame.point(x, y)
	if pixelmode then --Screen other than main
		if pixelmode == minigame.getColor() then
			love.graphics.circle("fill", x, y, minigame.pointSize, 32)
		end
	end
	
	if minigame.pointSize == 1 or minigame.pointSize == 2 then
		for xx = 1, minigame.pointSize do
			for yy = 1, minigame.pointSize do
				local nx, ny = minigame.x+x-1+xx-1, minigame.y+y-1+yy-1
				if minigame.onScissor(nx, ny) then
					setPixel(nx, ny, minigame.color)
				end
			end
		end
	else
		minigame.circle("fill", x, y, minigame.pointSize)
	end
end

-- ■ √
function minigame.setLineWidth(n)
	minigame.lineWidth = n or 1
end

-- ■ √
function minigame.setPointSize(n)
	minigame.pointSize = n or 1
end

-- ■ √
function minigame.print(txt, x, y, size, corner)
	if type(txt) ~= "string" then
		txt = tostring(txt)
	end
	if type(x) ~= "number" then
		error("\"minigame.print\" argument #2 must be a number")
	end
	if type(y) ~= "number" then
		error("\"minigame.print\" argument #3 must be a number")
	end
	
	local corner = false--corner or false
	--CURRENTLY DISABLED FEATURE, BUT I MAY TRY IT AGAIN IN THE FUTURE, SO I'M NOT GONNA REMOVE IT YET
	local size = size or 1
	local nx, ny = minigame.x, minigame.y
	
	local ww, hh = #fontdataquads["A"], #fontdataquads["A"][1]
	for letter = 1, string.len(txt) do
		local let = string.sub(txt, letter, letter)
		if let == "|" then
			minigame.print(string.sub(txt, letter+1, -1), x, y+hh*size, size)
			break
		elseif let ~= " " then
			for px = 1, ww do
				for py = 1, hh do
					for a = 1, size^2 do
						local d = math.ceil(a/size)
						
						local ax, ay = x+nx+(letter-1)*ww*size+px-2-size + (size-1)*(px-1) + (a-(d-1)*size), y+ny+py-1 + (size-1)*(py-1) + (d-1)
						if minigame.onScissor(ax, ay) or pixelmode then
							if fontdataquads[let] then
								local pix = fontdataquads[let][px][py]
								if pix then
									if not pixelmode then
										setPixel(ax, ay, math.max(0, pix-(1-minigame.color)))
									elseif pix == 1 and minigame.getColor() == pixelmode then
										if corner and a==1 then
											if px > 1 and py < hh and fontdataquads[let][px-1][py+1] == pix and fontdataquads[let][px-1][py] ~= pix and (py == 1 or fontdataquads[let][px-1][py-1] ~= pix) and (px == 2 or fontdataquads[let][px-2][py] ~= pix) then
												minigame.polygon("fill", ax-1, ay, ax-1, ay+size-1, ax-size, ay+size-1)
											elseif px < ww and py < hh and fontdataquads[let][px+1][py+1] == pix and fontdataquads[let][px+1][py] ~= pix and (py == 1 or fontdataquads[let][px+1][py-1] ~= pix) and (px == ww-1 or fontdataquads[let][px+2][py] ~= pix) then
												minigame.polygon("fill", ax+size-1, ay-1, ax+size*2-1, ay+size-1, ax+size-1, ay+size-1)
											end
											
											if px < ww and py < hh and fontdataquads[let][px+1][py+1] == pix and fontdataquads[let][px][py+1] ~= pix and (py == hh-1 or fontdataquads[let][px][py+2] ~= pix) and (px == 1 or fontdataquads[let][px-1][py+1] ~= pix) then
												minigame.polygon("fill", ax, ay+size-1, ax+size-1, ay+size-1, ax+size-1, ay+size*2-1)
											elseif py < hh and px > 1 and fontdataquads[let][px-1][py+1] == pix and fontdataquads[let][px][py+1] ~= pix and (py == hh-1 or fontdataquads[let][px][py+2] ~= pix) and (px == ww or fontdataquads[let][px+1][py+1] ~= pix) then
												minigame.polygon("fill", ax-1, ay+size-1, ax+size-1, ay+size-1, ax-1, ay+size*2-1)
											end
										end
										love.graphics.rectangle("fill", ax-nx, ay-ny, 1, 1)
									end
								end
							end
						end
						
					end
				end
			end
		end
	end
end

-- ■ √
function minigame.printSize(n, size)
	local size = size or 1
	local hs = 1
	for i = 1, string.len(n) do
		if string.sub(n, i, i) == "|" then hs = hs + 1 end
	end
	return #fontdataquads["A"]*string.len(n)*size, #fontdataquads["A"][1]*size*hs
end

-- ■ √
function minigame.printWidth(n, size)
	local ww, hh = minigame.printSize(n, size)
	return ww
end

-- ■ √
function minigame.printHeight(n, size)
	local ww, hh = minigame.printSize(n, size)
	return hh
end

-- ■ √
function minigame.numberSize(n, big, size)
	local big = big or false
	if big then
		big = "bignumbers"
	else
		big = "smallnumbers"
	end
	
	local size = size or 1
	
	local n = tostring(n)
	local s = n:split(":") --assuming time
	if #s == 2 then
		return #_G[big .. "dataquads"]["0"]*string.len(s[1])*size + #_G[big .. "dataquads"]["0"]*string.len(s[2])*size + minigame.printWidth(":")*size, #_G[big .. "dataquads"]["0"][1]*size
	else
		return #_G[big .. "dataquads"]["0"]*string.len(n)*size, #_G[big .. "dataquads"]["0"][1]*size
	end
end

-- ■ √
function minigame.numberWidth(n, big, size)
	local ww, hh = minigame.numberSize(n, big, size)
	return ww
end

-- ■ √
function minigame.numberHeight(n, big, size)
	local ww, hh = minigame.numberSize(n, big, size)
	return hh
end

-- ■ √
function minigame.number(txt, x, y, big, size)
	local big = big or false
	if big then
		big = "bignumbers"
	else
		big = "smallnumbers"
	end
	
	local size = size or 1
	
	if type(txt) ~= "string" then
		txt = tostring(txt)
	end
	
	local origtxt = txt --without the following gsubs:
	string.gsub(txt, ".", "")
	string.gsub(txt, "-", "")
	if not tonumber(txt) then
		--Let's assume it's a timer before throwing an error
		local s = txt:split(":")
		
		if #s == 2 then
			if tonumber(s[1]) and tonumber(s[2]) then
				local mm = math.abs( math.floor(tonumber(s[1])) )
				local ss = math.abs( math.floor(tonumber(s[2])) )
				
				minigame.number(mm, x, y, big, size)
				minigame.print(":", x+minigame.numberWidth(mm, big), y, size)
				minigame.number(ss, x+minigame.numberWidth(mm, big)+minigame.printWidth(":"), y, size)
				return
			end
		end
		
		-- Since it's a string, let's throw that to print
		minigame.print(origtxt, x, y, size)
	else
		--Then let's assume it's a decimal number
		local s = origtxt:split(".")
		
		if #s == 2 then
				print("YAY?")
			if tonumber(s[1]) and tonumber(s[2]) then
				--I may or may not have been lazy and copied this over, don't judge me
				
				local mm = math.abs( math.floor(tonumber(s[1])) )
				local ss = math.abs( math.floor(tonumber(s[2])) )
				
				minigame.number(mm, x, y, big, size)
				minigame.print(".", x+minigame.numberWidth(mm, big), y, size)
				minigame.number(ss, x+minigame.numberWidth(mm, big)+minigame.printWidth("."), y, size)
				return
			end
		end
	end
	
	if type(x) ~= "number" then
		error("\"minigame.number\" argument #2 must be a number")
	end
	if type(y) ~= "number" then
		error("\"minigame.number\" argument #3 must be a number")
	end
	
	local nx, ny = minigame.x, minigame.y
	for letter = 1, string.len(txt) do
		local let = string.sub(string.lower(txt), letter, letter)
		local ww, hh = #_G[big .. "dataquads"]["0"], #_G[big .. "dataquads"]["0"][1]
		
		for px = 1, ww do
			for py = 1, hh do
				for a = 1, size^2 do
					local d = math.ceil(a/size)
					
					local ax, ay = x+nx+(letter-1)*ww*size+px-3 + (size-1)*(px-1) + (a-(d-1)*size), y+ny+py-2 + (size-1)*(py-1) + (d-1)
					if minigame.onScissor(ax, ay) or pixelmode then
						if _G[big .. "dataquads"][let] then
							local pix = _G[big .. "dataquads"][let][px][py]
							if pix then
								if not pixelmode then
									setPixel(ax, ay, math.max(0, pix-(1-minigame.color)))
								elseif pix == 1 and minigame.getColor() == pixelmode then
									love.graphics.rectangle("fill", ax-nx, ay-ny, 1, 1)
								end
							end
						end
					end
					
				end
			end
		end
	end
end

-- ■ √
function minigame.setScissor(x, y, width, height)
	if pixelmode then
		love.graphics.setScissor(x, y, width, height)
		return
	end
	if x and y and width and height then
		minigame.scissorX = x
		minigame.scissorY = y
		minigame.scissorW = width
		minigame.scissorH = height
	else
		minigame.scissorX = 1
		minigame.scissorY = 1
		minigame.scissorW = minigame.width
		minigame.scissorH = minigame.height
	end
end

-- ■ √
function minigame.translate(x, y)
	if pixelmode then
		love.graphics.translate(x, y)
	end
	
	if x and y then
		minigame.x = x
		minigame.y = y
	else
		minigame.x = 1
		minigame.y = 1
	end
end

-- ■ ≈
function minigame.origin()
	if minigame.x ~= 1 or minigame.y ~= 1 then
		love.graphics.translate(-minigame.x, -minigame.y)
	end
	minigame.x = 1
	minigame.y = 1
end

-- □ √
function minigame.onScissor(x, y)
	if x >= minigame.scissorX and y >= minigame.scissorY and x <= minigame.scissorX+minigame.scissorW-1 and y <= minigame.scissorY+minigame.scissorH-1 then
		return minigame.onScreen(x, y)
	end
	return false
end

-- □ √
function minigame.onScreen(x, y)
	if x >= 1 and x <= minigame.width and y >= 1 and y <= minigame.height then
		return true
	end
	return false
end

-- ■ √
function minigame.setColor(color)
	minigame.color = minigame.fixedColor(color)
end

-- ■ √
function minigame.getColor()
	return minigame.color
end

-- ■ √
function minigame.setBackground(color)
	if color then
		minigame.background = minigame.fixedColor(color)
	else
		minigame.background = 0
	end
end

-- □ √
function imageToData(imgdata)
	if type(imgdata) == "table" then
		--let's hope it is formatted correctly...
		return imgdata
	end
	local ret = {}
	local retwidth = imgdata:getWidth()
	local retheight = imgdata:getHeight()
	for x = 1, imgdata:getWidth() do
		ret[x] = {}
		for y = 1, imgdata:getHeight() do
			local r, g, b, a = imgdata:getPixel(x-1, y-1)
			local color = (r+g+b)/3
			if a <= .125 then
				ret[x][y] = false
			else
				ret[x][y] = math.floor((1-color)*10/2)/10*2
			end
		end
	end
	return ret
end

-- □ √
function setPixel(x, y, c)
	local c = c or 0
	if minigame.blendmode == "replace" or pixel[x][y] == false or pixel[x][y] == 0 then
		pixel[x][y] = c
	else
		local ok, err = pcall(function()
		pixel[x][y] = math.min(1, pixel[x][y]+c)
		end)
		
		if not ok then
			print("Problem adding " .. tostring(c) .. " to pixel[" .. tostring(x) .. "][" .. tostring(y) .. "]")
		else
			pixel[x][y] = math.min(1, pixel[x][y]+c)
		end
	end
end
