self.name = "3D cube demo"
self.author = "alesan99"
local function line(x1, y1, x2, y2)
	--function by hugo, edited by me
	local pointsBetween = {}
	local w, h = math.abs(x1-x2), math.abs(y1-y2)
	local dist = math.max(w, h)
	local distDir = "h"
	if h > w then
		distDir = "v"
	end
	for i = 1, dist do
		local px, py
		local factx, facty = 1, 1
		if distDir == "h" then
			px = i
			py = math.floor((i/w)*h)
		else
			px = math.floor((i/h)*w)
			py = i
		end
		if x1 > x2 then factx = -1 end
		if y1 > y2 then facty = -1 end
		table.insert(pointsBetween, {x1+px*factx, y1+py*facty})
	end
	for j, w in pairs(pointsBetween) do
		minigame.point(round(w[1], 0), round(w[2], 0))
	end
end

--BY ALESAN99
--BIG THANKS TO http://petercollingridge.appspot.com/3D-tutorial

self.faceColor = {255, 255, 255}
self.lightVector = {0.5,-0.2,-2}

self.box1 = {faces = {{1,2,3,4},{8,7,6,5},
                    {1,4,8,5},{3,2,6,7},
                    {4,3,7,8}, {2,1,5,6}},
    nodes = {{-10,-10, 10},  {-10, 10, 10},
            { 10, 10, 10},  { 10,-10, 10},
            {-10,-10, -10}, {-10, 10, -10},
            { 10, 10, -10}, { 10,-10, -10}}
}

self.objects = {self.box1}


local function subtractVectors(v1, v2)
    return {v1[1] - v2[1],
            v1[2] - v2[2],
            v1[3] - v2[3]}
end

local function normaliseVector(v)
    local d = math.sqrt(v[1]*v[1]+v[2]*v[2]+v[3]*v[3])
    return {v[1]/d, v[2]/d, v[3]/d}
end

local function normalOfPlane(face, nodes)
    local n1 = nodes[face[1]]
    local n2 = nodes[face[2]]
    local n3 = nodes[face[3]]
    
    local v1 = subtractVectors(n1, n2)
    local v2 = subtractVectors(n1, n3)
    
    local v3 = {v1[2]*v2[3] - v1[3]*v2[2],
              v1[3]*v2[1] - v1[1]*v2[3],
              v1[1]*v2[2] - v1[2]*v2[1]}
              
    return v3
end

local function dotProduct(v1, v2)
    --Assume everything has 3 dimensions
    return v1[1]*v2[1] + v1[2]*v2[2] + v1[3]*v2[3]
end

self.currentX = false
self.currentY = false

local function translate3D(x, y, z, nodes)
    for i = 1, #nodes do
        nodes[i] = {nodes[i][1]+x, nodes[i][2]+y, nodes[i][3]+z}
    end
end

local function rotateX3D(theta, nodes)
    local ct = math.cos(theta)
    local st = math.sin(theta)
    local x, y, z
    
    for i = 1, #nodes do
        x = nodes[i][1]
        y = nodes[i][2]
        z = nodes[i][3]
        nodes[i] = {x, ct*y - st*z, st*y + ct*z}
    end
end


local function rotateY3D(theta, nodes)
    local ct = math.cos(theta)
    local st = math.sin(theta)
    local x, y, z

    for i = 1, #nodes do
        x = nodes[i][1]
        y = nodes[i][2]
        z = nodes[i][3]
        nodes[i] = {ct*x + st*z, y, -st*x + ct*z}
    end
end

self.lightVector = normaliseVector(self.lightVector)

rotateX3D(math.rad(30), self.box1.nodes)
rotateY3D(math.rad(60), self.box1.nodes)

self.colors = {0, .2, .4, .6, .8, 1}


function cube_load()

end

function cube_update(dt)
	rotateX3D(math.rad(dt*8), self.box1.nodes)
	rotateY3D(math.rad(dt*7.5), self.box1.nodes)
end

--[[All the drawing functions (so far):
	
	minigame.draw(imagedata, x, y)
	minigame.point(x, y)
	minigame.rectangle("fill" or "line", x, y, width, height)
	minigame.print(string, x ,y)
	minigame.setColor(0, 0.2, 0.4, 0.6, 0.8 or 1)
	minigame.setBackground(0, 0.2, 0.4, 0.6, 0.8, 1)
	
	minigame.setScissor(x, y, width, height)
	minigame.translate(x, y)
]]
function cube_draw()
	
	minigame.translate(17, 17)
    local i, o
    local face
    local nodes
    local node1
    local node2
    for j, w in pairs(self.objects) do
        local obj = w
        nodes = obj.nodes
        
        if obj.edges then
            local edges = obj.edges
        
            for i = 1, #obj.edges do
                node1 = nodes[edges[i][1]]
                node2 = nodes[edges[i][2]]
                line(node1[1], node1[2], node2[1], node2[2])
            end 
        end
        
		if obj.faces then
			for j, w in pairs(obj.faces) do
				face = w
				local fnorm = normalOfPlane(face, nodes)
				if fnorm[3] < 0 then
					local l = dotProduct(self.lightVector, normaliseVector(fnorm))
					minigame.setColor(6)
					line(nodes[face[1]][1], nodes[face[1]][2], nodes[face[2]][1], nodes[face[2]][2])
					line(nodes[face[2]][1], nodes[face[2]][2], nodes[face[3]][1], nodes[face[3]][2])
					line(nodes[face[3]][1], nodes[face[3]][2], nodes[face[4]][1], nodes[face[4]][2])
					line(nodes[face[4]][1], nodes[face[4]][2], nodes[face[1]][1], nodes[face[1]][2])
				end
			end
		end
	end
end

--[[All the buttons (so far):
	
	"R", "L", "U", "D": right, left, up and down
	"B": Big button/Action button
	"P": Pause button
]]
function cube_buttonPressed(button)
end

function cube_buttonReleased(button)
end

--[[All the game-related functions (so far):
	
	minigame.setLives(number)
	minigame.addLives(number)
	minigame.removeLives(number)
	
	minigame.gameOver() (automatically called when you have 0 lives)
	
	minigame.setScore(number) (must be a multiple of 100)
	minigame.addScore(number)
	minigame.removeScore(number)
	
	minigame.resetLevel()
	minigame.levelUp() (goes to level 1 if passed of level 10)
	
	minigame.buttonDown(button) (see list of buttons)
]]

--[[All the data-managing new functions (so far):
	
	table.copy(table)
	table.contains(table, entry)
	round(number, decimal digits)
	string.split(string, delimiter)
]]