self.name = "Filler"
self.icon = "filler_icon.png"
self.author = "HugoBDesigner"
self.maxLevel = 10

function filler_load(level)
	minigame.setProgress(0)
	level = math.max(minigame.getLevel(), level)
	minigame.saveGameSettings()
	
	self.gameover = {false, 4}
	self.levelup = {false, 1}
	
	self.reset = {false, 1}
	
	self.loadLevel(level)
end

function filler_update(dt)
	-- local speed = minigame.getSpeed()
	
	if self.gameover[1] then
		self.gameover[1] = self.gameover[1] + dt
		if self.gameover[1] >= self.gameover[2] then
			minigame.gameOver()
		end
		return
	end
	
	if self.levelup[1] then
		self.levelup[1] = self.levelup[1] + dt
		if self.levelup[1] >= self.levelup[2] then
			minigame.levelUp()
			self.levelup[1] = false
		end
		return
	end
	
	if minigame.buttonDown("big") then
		if not self.reset[1] then
			self.reset[1] = 0
		end
		self.reset[1] = self.reset[1] + dt
		
		if (self.reset[1] >= self.reset[2]) then
			minigame.resetLevel()
		end
	end
end

function filler_draw()
	minigame.pixelMode("b")
	
	local tilesize = math.floor( pixels / math.max(self.levelWidth, self.levelHeight) )
	local offx = math.floor( (pixelsH - self.levelWidth*tilesize) / 2 ) + 1
	local offy = math.floor( (pixelsV - self.levelHeight*tilesize) / 2 ) + 1
	
	for x = 1, self.levelWidth do
		for y = 1, self.levelHeight do
			local tile = self.level[x][y].tile
			local body = self.level[x][y].body
			local px = offx + (x-1)*tilesize
			local py = offy + (y-1)*tilesize
			local ps = tilesize
			
			
			if x == self.player.x and y == self.player.y then
				minigame.setColor(.6)
				if tilesize >= 3 then
					-- This code is cursed, I have no clue how it works, and I'm too afraid to touch it
					local r = tilesize / 2 - .5
					
					minigame.setColor(.4)
					minigame.rectangle("fill", px+1, py+1, ps-2, ps-2)
					
					minigame.pixelMode("r")
					minigame.setColor(.6)
					if tile == 2 then
						minigame.setColor(1)
					end
					minigame.circle("line", px+r-1, py+r-1, r)
					minigame.pixelMode("b")
				else
					minigame.rectangle("fill", px, py, ps, ps)
				end
			elseif body.any() then
				minigame.setColor(.6)
				if body.left then
					minigame.rectangle("fill", px, py+1, ps-1, ps-2)
				end
				if body.right then
					minigame.rectangle("fill", px+1, py+1, ps-1, ps-2)
				end
				if body.up then
					minigame.rectangle("fill", px+1, py, ps-2, ps-1)
				end
				if body.down then
					minigame.rectangle("fill", px+1, py+1, ps-2, ps-1)
				end
				if body.spawn then
					minigame.rectangle("fill", px+1, py+1, ps-2, ps-2)
				end
			elseif tile == 1 then -- Floor
				minigame.setColor(.2)
				if tilesize >= 3 then
					minigame.rectangle("fill", px+1, py+1, ps-2, ps-2)
				else
					minigame.rectangle("fill", px, py, ps, ps)
				end
			elseif tile == 2 then -- Target
				minigame.setColor(.4)
				if tilesize >= 3 then
					-- minigame.rectangle("line", px, py, ps, ps)
					minigame.setColor(.2)
					minigame.rectangle("fill", px+1, py+1, ps-2, ps-2)
				else
					minigame.rectangle("fill", px, py, ps, ps)
				end
			end
			
			if tile == 2 then
				minigame.setColor(.4)
				if tilesize >= 3 then
					minigame.rectangle("line", px, py, ps, ps)
				end
			end
		end
	end
	
	if minigame.isPaused() then
		minigame.setColor(.6)
		minigame.rectangle("line", 1, 1, 32, 32)
		minigame.setColor(.4)
		minigame.rectangle("line", 2, 2, 30, 30)
		minigame.setColor(.2)
		minigame.rectangle("line", 3, 3, 28, 28)
	end
	
	if self.gameover[1] or self.levelup[1] then
		minigame.pixelMode("r")
		local f
		
		local off = false
		if self.gameover[1] then
			f = self.gameover[1]/(self.gameover[2]/2)
			if self.gameover[1] >= self.gameover[2]/2 then
				f = 1 - (self.gameover[1]-self.gameover[2]/2)/(self.gameover[2]/2)
				off = true
			end
		elseif self.levelup[1] then
			f = self.levelup[1]/(self.levelup[2]/2)
			if self.levelup[1] >= self.levelup[2]/2 then
				f = 1 - (self.levelup[1]-self.levelup[2]/2)/(self.levelup[2]/2)
				off = true
			end
		end
		if off then
			minigame.setColor(0)
			minigame.rectangle("fill", 1, 1, 32, 32)
		end
		minigame.setColor(1)
		minigame.rectangle("fill", 1, 32-math.floor(f*32), 32, 32)
	end
end

function filler_buttonPressed(button)
	if self.levelup[1] then return end
	
	if button == "big" and minigame.buttonDown("pause") then
		minigame.levelUp()
	end
	
	-- Will treat big as undo
	if button == "big" then
		local body = self.level[self.player.x][self.player.y].body
		if not body.spawn then
			button = (
				body.left and "left" or
				body.right and "right" or
				body.up and "up" or
				body.down and "down"
			)
		end
	end
	
	local nextPos = {x = 0, y = 0}
	
	local opposite = (
		button == "left" and "right" or
		button == "right" and "left" or
		button == "up" and "down" or
		button == "down" and "up"
	)
	
	if button == "up" then
		nextPos.x = self.player.x
		nextPos.y = self.player.y-1
	elseif button == "down" then
		nextPos.x = self.player.x
		nextPos.y = self.player.y+1
	elseif button == "left" then
		nextPos.x = self.player.x-1
		nextPos.y = self.player.y
	elseif button == "right" then
		nextPos.x = self.player.x+1
		nextPos.y = self.player.y
	end
	
	if nextPos.x >= 1 and nextPos.y >= 1 and nextPos.x <= self.levelWidth and nextPos.y <= self.levelHeight then -- In bounds
		local curTile = self.level[self.player.x][self.player.y]
		local nextTile = self.level[nextPos.x][nextPos.y]
		
		local movePlayer = false
		if nextTile.tile ~= 0 then -- Not wall
			if nextTile.body[opposite] and curTile.body[button] then -- It's an undo
				nextTile.body[opposite] = false
				curTile.body[button] = false
				movePlayer = true
			elseif not (nextTile.body.any()) then -- No wall and no body means you can move
				nextTile.body[opposite] = true
				curTile.body[button] = true
				movePlayer = true
			end
		end
		
		if (movePlayer) then
			self.player.x = nextPos.x
			self.player.y = nextPos.y
			self.checkWin()
		end
	end
end

function filler_buttonReleased(button)
	if self.flipside == button then
		self.flipside = false
	end
end

function filler_gameOver()
	self.gameover[1] = 0
end

function self.checkWin()
	local hasWon = true
	
	local playerx, playery = self.player.x, self.player.y
	local playertile = self.level[playerx][playery].tile
	
	if (playertile ~= 2) then
		-- Can only win if the player is in a target
		hasWon = false -- Used to return false, but now we check for progress
	end
	
	local totalTiles = 0
	local visitedTiles = 0
	for x = 1, self.levelWidth do
		for y = 1, self.levelHeight do
			local tile = self.level[x][y].tile
			local body = self.level[x][y].body.any()
			
			if tile ~= 0 then
				totalTiles = totalTiles + 1
				if body then
					visitedTiles = visitedTiles + 1
				else
					-- Unvisited tile means no victory
					hasWon = false
				end
			end
		end
	end
	
	local progress = 19/20 * (visitedTiles / totalTiles) -- We lie to the progress meter
	minigame.setProgress( progress )
	
	if hasWon then
		minigame.setProgress( 1 )
		print("Victory!")
		minigame.addScore(1000)
		if minigame.getLevel() == self.maxLevel then
			self.gameover[1] = 0
		else
			self.levelup[1] = 0
		end
	end
end

function self.loadLevel(n)
	local leveltext = self.levels[n]
	leveltext = string.gsub(leveltext, "\r\n", "\n")
	
	self.level = {}
	self.player = {}
	
	self.levelWidth = 0
	local lines = leveltext:split("\n")
	self.levelHeight = #lines - 2
	
	-- We ignore the first line, the last line, the first column, and the last column
	for idx, line in ipairs(lines) do
		if idx == 1 then
			self.levelWidth = string.len(line) - 2
			for x = 1, self.levelWidth do
				self.level[x] = {}
			end
		elseif idx ~= #lines then
			local y = idx-1
			for pos = 2, string.len(line)-1 do
				local x = pos-1
				local char = string.sub(line, pos, pos)
				
				self.level[x][y] = {tile = 1, body = {left = false, right = false, up = false, down = false}} -- 1 for floor
				self.level[x][y].body.any = function()
					return self.level[x][y].body.left or self.level[x][y].body.right or self.level[x][y].body.up or self.level[x][y].body.down
				end
				if char == "p" then
					-- Player spawn
					self.player = {x = x, y = y}
					self.level[x][y].body.spawn = true -- Just for easier reference
				elseif char == "#" then
					self.level[x][y].tile = 0 -- 0 for wall
				elseif char == "." then
					self.level[x][y].tile = 1 -- 1 for floor
				elseif char == "o" then
					self.level[x][y].tile = 2 -- 2 for target
				end
			end
		end
	end
end

self.levels = {
-- message [1/10] "Tutorial"
[[#########
#p..#####
###.#####
#...#...#
#.###.#.#
#.....#o#
#########]]

, -- message [2/10] "Baby Steps"

[[#######
#p..###
#.....#
#.##..#
#.##..#
#....o#
#######]]

, -- message [3/10] "Free Choice"

[[#######
#..p..#
#.....#
#.o#o.#
#..#..#
#..#..#
#######]]

, -- message [4/10] "The Deceiver"

[[#######
#..#..#
#.o...#
#.....#
#.....#
#..po.#
#######]]

, -- message [5/10] "Runny Nose"
-- message Designed by Demon Arisen

[[#######
#.....#
#.o.o.#
#..p..#
#.....#
#.###.#
#.....#
#######]]

, -- message [6/10] "Far Loop"

[[##########
#####....#
#.....o..#
#.##....##
#.#..p...#
#.#......#
#.#......#
#...##...#
##########]]

, -- message [7/10] "Inside-Out"

[[########
#......#
#......#
#..po..#
#..##..#
#..#...#
#......#
#.....##
########]]

, -- message [8/10] "Serpentine"

[[############
##.........#
#....opo...#
#.....#...##
###...#...##
###...#..###
###......###
############]]

, -- message [9/10] "Dicey"
-- message Designed by Demon Arisen

[[###########
#.........#
#.#..#..#o#
#.........#
#.........#
#.........#
#.#..#..#.#
#.p.......#
###########]]

, --message [10/10] "Grated Cheese"

[[############
####o...####
####....####
#........###
#....#.....#
#..#..p..#.#
#....#.....#
###....#..##
###.......##
######....##
############]]
}
