self.name = "Tetris"
self.icon = "tetris_icon.png"
self.author = "HugoBDesigner"
minigame.setLives(1)

self.instdown = false
self.onebyone = false
self.preview = false
self.maxScore = 5000
self.maxLevel = 10
function tetris_load(level)
	minigame.setProgress(0)
	minigame.setScore(0)
	
	self.dead = false
	self.deathTime = 1
	self.levelup = {false, 1}
	
	self.special = {false, 1}
	
	self.lines = {}
	self.lineAnim = {false, 2}
	
	self.settings = {
		{"Maximum score", "maxScore", "number", {3000, 8000, 500}},
		{"Instant downpress", "instdown", "toggle"},
		{"Special 1x1 pieces", "onebyone", "toggle"},
		{"Preview tile position", "preview", "toggle"}
	}
	
	self.pieces = {
		{
			0, 0, 0,
			0, 0, 1,		--   #
			1, 1, 1			-- ###
		},
		{
			0, 0, 0,
			1, 0, 0,		-- #
			1, 1, 1			-- ###
		},
		{
			0, 0, 0,
			0, 1, 0,		--  #
			1, 1, 1			-- ###
		},
		{
			0, 0, 0,
			1, 1, 0,		-- ##
			0, 1, 1			--  ##
		},
		{
			0, 0, 0,
			0, 1, 1,		--  ##
			1, 1, 0			-- ##
		},
		{
			0, 0, 0, 0,
			0, 0, 0, 0,
			1, 1, 1, 1,		-- ####
			0, 0, 0, 0
		},
		{
			1, 1,			-- ##
			1, 1			-- ##
		},
		{
			1				-- #
		}
	}
	
	self.tiles = {}
	self.down = {false, .3, .05}
	self.big = {false, .3, .05}
	self.up = {false, .3, .05}
	self.left = {false, .3, .05}
	self.right = {false, .3, .05}
	
	--	screen - next piece - border - line - innerborder - left border
	for x = 1, minigame.width-4-2-2-1-2-4 do
		self.tiles[x] = {}
		for y = 1, minigame.height do
			self.tiles[x][y] = 0
		end
	end
	
	
	print("level = " .. level)
	if level > 1 then
		local n = level-1
		-- 3 to 8 pieces * level
		n = n * (3+math.random(6)-1)
		
		local tt = {}
		for y = 1, #self.tiles[1] do
			tt[y] = {}
			for x = 1, #self.tiles do
				table.insert(tt[y], x)
			end
		end
		
		local yy = #self.tiles[1]
		while n > 0 do
			n = n-1
			
			local t = math.random(#tt[yy])
			self.tiles[ tt[yy][t] ][ yy ] = 1
			table.remove(tt[yy], t)
			
			if math.random() <= .05 then --5% chance random pieces will go up a step
				yy = math.max(1, yy-1)
			end
			
			if #tt[yy] == 1 then
				yy = math.max(1, yy-1)
			end
			if yy == 1 then
				n = 0
			end
		end
	end
	
	self.nextPiece = {math.random(#self.pieces-1), math.random(4)-1}
	self.nextPiece[3] = self.rotatedPiece(self.nextPiece[1], self.nextPiece[2])
	
	self.falling = {
		timer = 0,
		maxtime = 1
	}
	
	self.gameover = {false, 4}
	
	self.newTile()
end

function tetris_update(dt)
	local speed = minigame.getSpeed()
	
	if self.gameover[1] then
		self.gameover[1] = self.gameover[1] + dt
		if self.gameover[1] >= self.gameover[2] then
			minigame.gameOver()
		end
		return
	end
	
	if self.levelup[1] then
		self.levelup[1] = self.levelup[1] + dt
		if self.levelup[1] >= self.levelup[2] then
			minigame.levelUp()
			self.levelup[1] = false
		end
		return
	end
	
	if self.special[1] then
		self.special[1] = self.special[1] + dt
		while self.special[1] >= self.special[2] do
			self.special[1] = self.special[1] - self.special[2]
		end
	end
	
	if self.lineAnim[1] then
		self.lineAnim[1] = self.lineAnim[1] + dt
		if self.lineAnim[1] >= self.lineAnim[2] then
			self.lineAnim[1] = false
			
			for i = #self.lines, 1, -1 do
				local v = self.lines[i]
				
				for x = 1, #self.tiles do
					for y = v, 1, -1 do
						self.tiles[x][y] = self.tiles[x][y-1] or 0
					end
				end
				
			end
			
			if #self.lines == 1 then
				minigame.addScore(100)
			elseif #self.lines == 2 then
				minigame.addScore(200)
			elseif #self.lines == 3 then
				minigame.addScore(400)
			else
				minigame.addScore(1000)
			end
			
			minigame.setProgress(math.min(1, minigame.getScore()/self.maxScore))
			
			self.lines = {}
			
			if minigame.getScore() >= self.maxScore then
				if minigame.getLevel() == self.maxLevel then
					self.gameover[1] = 0
				else
					self.levelup[1] = 0
				end
			end
		end
		return
	end
	
	self.falling.timer = self.falling.timer + dt*speed
	
	if self.down[1] then
		self.falling.timer = 0
		self.down[1] = self.down[1] - dt
		if self.down[1] <= 0 then
			self.down[1] = self.down[3]
			if not self.instdown then
				self.movePiece(0, 1)
			end
		end
	end
	if self.left[1] then
		self.left[1] = self.left[1] - dt
		if self.left[1] <= 0 then
			self.left[1] = self.left[3]
			self.movePiece(-1, 0)
		end
	end
	if self.right[1] then
		self.right[1] = self.right[1] - dt
		if self.right[1] <= 0 then
			self.right[1] = self.right[3]
			self.movePiece(1, 0)
		end
	end
	
	if self.big[1] then
		self.big[1] = self.big[1] - dt
		if self.big[1] <= 0 then
			self.big[1] = self.big[3]
			self.movePiece(0, -1)
		end
	elseif self.up[1] then
		self.up[1] = self.up[1] - dt
		if self.up[1] <= 0 then
			self.up[1] = self.up[3]
			self.movePiece(0, -1)
		end
	end
	
	if self.falling.timer >= self.falling.maxtime then
		self.falling.timer = 0
		self.movePiece(0, 1)
	end
end

function tetris_draw()
	------------------
	-- PLACED TILES --
	------------------
	for x = 1, #self.tiles do
		for y = 1, #self.tiles[x] do
			minigame.setColor(1)
			if table.contains(self.lines, y) then
				minigame.setColor(.8)
				local f = self.lineAnim[1]/self.lineAnim[2]*4
				f = f - math.floor(f)
				if f <= .5 then
					minigame.setColor(.2)
				end
			end
			if self.tiles[x][y] == 1 then
				minigame.point(x+2, y)
			end
		end
	end
	
	
	-------------
	-- BORDERS --
	-------------
	minigame.setColor(.2)
	minigame.line(#self.tiles+3, 1, #self.tiles+3, 32)
	minigame.line(2, 1, 2, 32)
	minigame.line(#self.tiles+5, 1, #self.tiles+5, 32)
	minigame.line(1, 1, 1, 32)
	minigame.setColor(.4)
	minigame.line(#self.tiles+4, 1, #self.tiles+4, 32)
	minigame.setColor(.6)
	
	
	------------------
	-- TILE PREVIEW --
	------------------
	if self.preview then
		minigame.setColor(.2)
		local yadd = 0
		while not self.collide(0, yadd) do
			yadd = yadd+1
		end
		
		local i = 1
		local s = math.sqrt(#self.pieces[self.falling.tile])
		for y = 1, s do
			for x = 1, s do
				local xx, yy = self.falling.x + (x-1), self.falling.y + (y-1) + yadd - 1
				if self.falling.rottile[i] == 1 then
					minigame.point(xx+2, yy)
				end
				i = i+1
			end
		end
	end
	
	minigame.setColor(.6)
	---------------
	-- NEXT TILE --
	---------------
	local i = 1
	local s = math.sqrt(#self.pieces[ self.nextPiece[1] ])
	for y = 1, s do
		for x = 1, s do
			local xx, yy = (#self.tiles + 3), 3 + (y-1)
			xx = xx + math.floor((32-xx)/2 - s/2) + (x-1)
			if self.nextPiece[3][i] == 1 then
				minigame.point(xx+2, yy)
			end
			
			i = i+1
		end
	end
	
	
	-------------------
	-- FALLING PIECE --
	-------------------
	minigame.setColor(.8)
	if self.special[1] then
		local f = self.special[1]/self.special[2]
		f = math.floor(f*4*2)/2
		if math.mod(f, 2) ~= 0 and f - math.floor(f) == 0 then
			f = 1
		else
			f = f - math.floor(f)
		end
		
		minigame.setColor(.2+f*2/5)
	end
	
	local i = 1
	local s = math.sqrt(#self.pieces[self.falling.tile])
	for y = 1, s do
		for x = 1, s do
			local xx, yy = self.falling.x + (x-1), self.falling.y + (y-1)
			if self.falling.rottile[i] == 1 then
				minigame.point(xx+2, yy)
			end
			i = i+1
		end
	end
	
	
	------------
	-- PAUSED --
	------------
	minigame.pixelMode("b")
	if minigame.isPaused() then
		minigame.setColor(.6)
		minigame.rectangle("line", 1, 1, 32, 32)
		minigame.setColor(.4)
		minigame.rectangle("line", 2, 2, 30, 30)
		minigame.setColor(.2)
		minigame.rectangle("line", 3, 3, 28, 28)
	end
	
	
	---------------
	-- GAME OVER --
	---------------
	if self.gameover[1] or self.levelup[1] then
		minigame.pixelMode("r")
		local f
		
		local off = false
		if self.gameover[1] then
			f = self.gameover[1]/(self.gameover[2]/2)
			if self.gameover[1] >= self.gameover[2]/2 then
				f = 1 - (self.gameover[1]-self.gameover[2]/2)/(self.gameover[2]/2)
				off = true
			end
		elseif self.levelup[1] then
			f = self.levelup[1]/(self.levelup[2]/2)
			if self.levelup[1] >= self.levelup[2]/2 then
				f = 1 - (self.levelup[1]-self.levelup[2]/2)/(self.levelup[2]/2)
				off = true
			end
		end
		if off then
			minigame.setColor(0)
			minigame.rectangle("fill", 1, 1, 32, 32)
		end
		minigame.setColor(1)
		minigame.rectangle("fill", 1, 32-math.floor(f*32), 32, 32)
	end
end

function tetris_buttonPressed(button)
	if self.dead or self.levelup[1] then return end
	
	if button == "down" then
		self.falling.timer = 0
		if self.instdown then
			local y = 0
			while not self.collide(0, y) do
				y = y+1
			end
			self.falling.y = self.falling.y+y-1
			self.dropPiece()
		else
			self.down[1] = self.down[2]
			self.movePiece(0, 1)
		end
	elseif button == "big" then
		self.big[1] = self.big[2]
		self.movePiece(0, -1)
	elseif button == "up" then
		self.up[1] = self.up[2]
		self.movePiece(0, -1)
	elseif button == "left" then
		self.left[1] = self.left[2]
		self.movePiece(-1, 0)
	elseif button == "right" then
		self.right[1] = self.right[2]
		self.movePiece(1, 0)
	end
end

function tetris_buttonReleased(button)
	if button == "down" then
		self.down[1] = false
	elseif button == "up" then
		self.up[1] = false
	elseif button == "big" then
		self.big[1] = false
	elseif button == "left" then
		self.left[1] = false
	elseif button == "right" then
		self.right[1] = false
	end
end

function self.movePiece(x, y)
	if self.lineAnim[1] then return end
	if y < 0 then
		local old = self.falling.rot
		self.falling.rot = self.falling.rot + 1
		if self.falling.rot > 3 then
			self.falling.rot = 0
		end
		self.falling.rottile = self.rotatedPiece()
		if self.collide(0, 0) then
			self.falling.rot = old
			self.falling.rottile = self.rotatedPiece()
		end
	elseif y > 0 then
		if self.collide(0, 1) then
			self.dropPiece()
		else
			self.falling.y = self.falling.y + 1
		end
	end
	
	if x < 0 then
		if not self.collide(-1, 0) then
			self.falling.x = self.falling.x-1
		end
	elseif x > 0 then
		if not self.collide(1, 0) then
			self.falling.x = self.falling.x+1
		end
	end
end

function self.dropPiece()
	local s = math.sqrt(#self.falling.rottile)
	local x, y = 1, 1
	for i = 1, #self.falling.rottile do
		if self.falling.rottile[i] == 1 then
			if self.falling.y+(y-1) < 1 then
				self.gameover[1] = 0
			else
				self.tiles[ self.falling.x+(x-1) ][ self.falling.y+(y-1) ] = 1
			end
		end
		
		x = x+1
		if x > s then
			x = 1
			y = y+1
		end
	end
	self.checkLine()
	
	self.newTile()
end

function self.newTile()
	self.falling.tile = self.nextPiece[1]
	local s = math.sqrt(#self.pieces[self.falling.tile])
	self.falling.x = math.ceil(#self.tiles/2 - s/2)
	self.falling.timer = 0
	self.falling.rot = self.nextPiece[2]
	
	self.falling.rottile = self.rotatedPiece()
	
	self.special[1] = false
	if self.falling.tile == #self.pieces then
		self.special[1] = 0
	end
	
	--[[local h = 0
	for y = 1, s do
		for x = 1, s do
			if self.falling.rottile[(y-1)*s+x] == 1 then
				h = h+1
				break
			end
		end
	end]]
	self.falling.y = 1
	if self.collide(0, 0) then
		self.gameover[1] = 0
	end
	
	local off = 1
	if self.onebyone then off = 0 end
	self.nextPiece[1] = math.random(#self.pieces-off)
	self.nextPiece[2] = math.random(4)-1
	self.nextPiece[3] = self.rotatedPiece(self.nextPiece[1], self.nextPiece[2])
end

function self.collide(ex, ey)
	local x, y = 1, 1
	for i = 1, #self.falling.rottile do
		
		local xx = self.falling.x + (x-1) + ex
		local yy = self.falling.y + (y-1) + ey
		
		if self.falling.rottile[i] == 1 then
			if yy > #self.tiles[1] then
				return true
			elseif xx < 1 or xx > #self.tiles then
				return true
			elseif self.tiles[xx][yy] == 1 then
				if self.falling.tile == #self.pieces then
					--GHOST BLOCK!
					local hasgap = false
					for gy = #self.tiles[xx], yy, -1 do
						if self.tiles[xx][gy] == 0 then
							hasgap = true
						end
					end
					if not hasgap then return true end
				else
					return true
				end
			end
		end
		
		x = x+1
		if x > math.sqrt(#self.falling.rottile) then
			x = 1
			y = y+1
		end
	end
	return false
end

function self.rotatedPiece(ref, rot)
	local ref = ref or self.falling.tile
	local rot = rot or self.falling.rot
	
	if ref > #self.tiles then
		return {1}
	end
	
	local ret = {}
	for a = 1, #self.pieces[ref] do
		table.insert(ret, self.pieces[ref][a])
	end
	local function rotate()
		local nret = {}
		local s = math.sqrt(#ret)
		
		local x, y = 1, 1
		for i = 1, #ret do
			local v = i + (s^2 - s) - (x-1)*(s+1) - (y-1)*(s-1)
			nret[i] = ret[v]
			x = x+1
			if x > s then
				x = 1
				y = y+1
			end
		end
		
		ret = nret
	end
	
	local n = rot
	while n > 0 do
		rotate()
		n = n-1
	end
	
	return ret
end

function self.checkLine()
	for y = #self.tiles[1], 1, -1 do
		local hasleak = false
		for x = 1, #self.tiles do
			if self.tiles[x][y] == 0 then
				hasleak = true
				break
			end
		end
		
		if not hasleak then
			table.insert(self.lines, y)
			self.lineAnim[1] = 0
		end
	end
end
