--[[
	MINIGAME SIMULATOR 0.4
		by HugoBDesigner
	
	Licensed under the Creative Commons 3.0 Share-Alike terms.
		http://creativecommons.org/licenses/by-nc-sa/3.0/
	
	You're entirely free to make/sell/share/do whatever you want with files that run on this game.
	BUT you CANNOT sell/share/do whatever you want with the game itself or with content created
	by someone else without its author's permission AND without proper crediting.
	
	╔═══════════════════════════════════════════════╗
	║ Changelog:                                    ║
	╠═══════════════════════════════════════════════╣
	║ Version 0.4                                   ║
	║  - Converted to LÖVE 0.11.4;                  ║
	║  - Implemented a new casing skin;             ║
	║  - Added game "Filler";                       ║
	║  - Added options for custom pixel size;       ║
	║  - Added options for screen scaling;          ║
	║  - Fixed some minor bugs.                     ║
	╠═══════════════════════════════════════════════╣
	║ Version 0.3                                   ║
	║  - Fixed several bugs;                        ║
	║  - Rewrote sets loading system;               ║
	║  - Improved game controls;                    ║
	║  - Added several new minigame functions;      ║
	║  - Added custom settings loading for sets;    ║
	║  - Added customizable buttons for sets;       ║
	║  - Added multiple screens support;            ║
	║  - Added new drawing functions;               ║
	║  - Added pixel blending for transparency;     ║
	║  - Fixed several bugs with Snake;             ║
	║  - Converted to LÖVE 0.10.1;                  ║
	║  - Improved default game set;                 ║
	║  - Added lots of new game and sets functions; ║
	║  - Added game "Tetris";                       ║
	║  - Added demo "3D Cube" by alesan99;          ║
	║  - Added a game selection menu;               ║
	║  - Added a new game set: "Scribble";          ║
	║  - Rewrote data loading/saving system;        ║
	║  - Improved game loading;                     ║
	║  - Removed game speed as default on update.   ║
	╠═══════════════════════════════════════════════╣
	║ Version 0.2                                   ║
	║  - Added pause;                               ║
	║  - Added settings;                            ║
	║  - Custom settings per game;                  ║
	║  - Custom skin and minigame colors;           ║
	║  - Added scores and highscores;               ║
	║  - Converted to LÖVE 0.9.1.                   ║
	╠═══════════════════════════════════════════════╣
	║ Version 0.1                                   ║
	║ Changelog:                                    ║
	║  - Started project.                           ║
	╚═══════════════════════════════════════════════╝
	╔══════════════════════════════════════════════════════╗
	║ TO DO:                                               ║
	║  - Custom error handler;                             ║
	║  - Add sounds;                                       ║
	║  - Game sorting and categorization systems;          ║
	║  - Online game downloader;                           ║
	║  - More pre-made games.                              ║
	╚══════════════════════════════════════════════════════╝
	
]]

minigame = {}
function love.load()
	pixels = 32
	pixelsH, pixelsV = 32, 32

	require "class"
	require "draw"
	require "menu"
	
	seed = false
	
	minigame.caption = "Minigame Simulator 0.4"
	
	if love.filesystem.getInfo("console.txt") then
		local s = love.filesystem.read("console.txt")
		if s == "true" then
			if love._openConsole then
				love._openConsole()
			end
		end
	end
	
	fontglyphs = "abcdefghijklmnopqrstuvwxyz0123456789.!?#&()[]{}" .. "LRUDOXS" .. "ABCDEFG"
	--	Upper cases are:	► ◄ ▲ ▼ ○ × □		:) :( :D :o :| ;) :P
	
	regularglyphs = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789.,:!?()[]{}/\\*;><+-_='\""
	regularfont = love.image.newImageData("regularfont.png")
	regularfontquads = {}
	for i = 1, string.len(regularglyphs) do
		local c = string.sub(regularglyphs, i, i)
		regularfontquads[c] = {}
		for x = 0, 5 do
			for y = 0, 10 do
				local px = (i-1)*6+x
				local cr, cg, cb, ca = regularfont:getPixel(px, y)
				if ca > 128 then
					table.insert(regularfontquads[c], {x, y})
				end
			end
		end
	end
	
	games = love.filesystem.getDirectoryItems("games") --All the games you can have
	gamefiles = love.filesystem.getDirectoryItems("games")
	
	quickTest = false--"snake"
	
	state = "game"
	if not quickTest then
		state = "menu"
	end
	currentgame = false
	lastgame = false
	gamelevel = 1
	gamespeed = 1
	gamelives = 1
	gamescore = 0
	gametime = false
	gameprogress = 0
	gamehighscore = 0
	gamehighscores = {}
	
	font = love.graphics.newFont(12)
	
	local delete = {}
	if #games >= 1 then
		for i = 1, #games do
			print("gamefiles[" .. i .. "] = " .. tostring(gamefiles[i]))
			if string.sub(games[i], -4, -1) == ".lua" then
				local current = string.sub(games[i], 1, -5)
				if #games >= i + 1 then
					for a = #games, 1, -1 do
						if string.sub(games[a], 1, -5) == current and a ~= i then --DELETE FILES WITH SIMILAR NAME TO THE GAME
							table.insert(delete, a)
						end
					end
				end
				gamefiles[i] = games[i]
				games[i] = current
			else --DELETE NON-CODE FILES FROM GAMES LIST
				table.insert(delete, i)
			end
		end
	else
		games = {}
	end
	
	table.sort(delete)
	local prev = false
	for i = #delete, 1, -1 do
		local v = delete[i]
		if v ~= prev then --AVOID REPETITION
			table.remove(games, v)
			table.remove(gamefiles, v)
		end
		prev = v
	end
	
	table.sort(games)
	table.sort(gamefiles)
	
	for i, v in ipairs(gamefiles) do
		print("gamefiles[" .. i .. "] = " .. tostring(gamefiles[i]))
	end
	love.graphics.setBackgroundColor(0, 0, 0, 1)
	love.graphics.setDefaultFilter("nearest", "nearest")
	
	---------------------
	-- Window Settings --
	---------------------
	gamewidth = 0
	gameheight = 0
	love.window.setTitle(minigame.caption)
	
	
	button_shapes = {"circle", "rectangle", "roundrectangle", "ellipse", "arrowup", "arrowleft", "arrowdown", "arrowright"}
	buttonsDown = {}
	
	----------
	--IMAGES--
	----------
	minigame.skins = love.filesystem.getDirectoryItems("sets")
	minigame.set = "default"
	minigame.skincolor = {55, 205, 255}
	require "settings"
	
	gamepaused = false
	pixelmode = false
	love.graphics.setLineWidth(1)
	
	lastlpress = 0
	lastrpress = 0
	
	pixel = {}
	
	for x = 1, pixelsH do
		pixel[x] = {}
		for y = 1, pixelsV do
			pixel[x][y] = false
		end
	end
	
	
	-----------------------
	--CUSTOM SKIN LOADING--
	-----------------------
	minigame.loadSettings()
	
	menut.games = {}
	for i, v in ipairs(games) do
		minigame.loadGame(v)
		if _G[v .. "_data"].name then
			menut.games[v] = _G[v .. "_data"].name
		else
			menut.games[v] = v
		end
	end
	
	--Sort games alphabetically. I'm sure there's a simpler way of doing this, but this will serve for now...
	table.sort(games, function(a, b)
		return alphabetic_order(menut.games[a], menut.games[b])
	end)
	
	print("Games organized alphabetically!")
	for i, v in ipairs(games) do
		print("Game \"" .. i .. "\" name = " .. tostring(menut.games[v]))
		print("games[" .. i .. "] = " .. tostring(games[i]))
	end
	
	
	loadSet(minigame.set, true)
	
	--intro()
	
	if quickTest then
		loadGame(quickTest)
	else
		_G[state .. "_load"]()
	end
end

function prepareGame()
	print("PREPAREGAME")
	minigame.pixelMode("r")
	
	local chunk = love.filesystem.load("games/" .. currentgame .. ".lua")
	local ok, vartable = pcall(chunk)
	if ok then
		chunk()
	end
	
	--minigame.loadSettings()
end

function loadGameSettings(game)
	collectgarbage()
	print("LOADGAMESETTINGS")
	local n = table.contains(games, game)
	if type(game) == "number" then
		n = game
	end
	if n then
		self = _G[games[n] .. "_data"]
		love.filesystem.load("games/" .. gamefiles[n])()
		--LOAD MAIN SETTINGS
	else
		minigame.setLives(5)
		minigame.setScore(0)
		minigame.setTime(false)
		minigame.setProgress(0)
	end
end

function love.update(dt)
	dt = math.min(dt, 1/30)
	if state == "game" then
		if _G[currentgame .. "_update"] and not gamepaused then
			_G[currentgame .. "_update"](dt)
		end
		
		if gametime then
			gametime = math.max(0, gametime - dt)
			if gametime == 0 then
				if _G[currentgame .. "_timeUp"] then
					_G[currentgame .. "_timeUp"]()
				else
					minigame.timeUp()
				end
			end
		end
	else
		_G[state .. "_update"](dt)
	end
	
	--reset screen so nothing stays
	minigame.clearScreen()
	
	--Draw background
	minigame.setColor(minigame.background)
	minigame.rectangle("fill", 1, 1, 32, 32)
	
	--Draw game
	if state == "game" and currentgame then
		pixelmode = false
		if _G[currentgame .. "_draw"] then
			_G[currentgame .. "_draw"]()
		end
	end
end

function love.draw()
	love.graphics.setColor(1, 1, 1, 1)
	if minigamebackgroundimg then
		love.graphics.draw(minigamebackgroundimg, 0, 0)
	end
	local r, g, b = unpack(minigame.skincolor)
	love.graphics.setColor(r/255, g/255, b/255, 1)
	love.graphics.draw(minigameimg, 0, 0)
	love.graphics.setColor(1, 1, 1, 1)
	if minigameoverlayimg then
		love.graphics.draw(minigameoverlayimg, 0, 0)
	end
	
	for i, v in pairs(minigame.buttons) do
		if v.down then
			love.graphics.draw(v.imagedown, v.x-v.imagedown:getWidth()/2, v.y-v.imagedown:getHeight()/2)
		else
			love.graphics.draw(v.image, v.x-v.image:getWidth()/2, v.y-v.image:getHeight()/2)
		end
	end
	
	for i, v in pairs(minigame.screens) do
		love.graphics.setColor(1, 1, 1, 1)
		love.graphics.draw(v.off, v.x, v.y)
		for a = 1, 5 do
			if i == "main" and state == "game" then
				pixelmode = false
				local c = a/5
				love.graphics.setColor(1, 1, 1, c)
				
				local posx, posy = minigame.screens.main.x, minigame.screens.main.y
				local pixelsX, pixelsY = minigame.screens.main.offsetX, minigame.screens.main.offsetY
				local pixelSize, pixelOffset = minigame.screens.main.pixelSize, minigame.screens.main.pixelOffset
				local function myStencil()
					love.graphics.clear()
					for x = 1, pixelsH do
						for y = 1, pixelsV do
							local p = math.floor(pixel[x][y]*5)
							local cc = math.floor(c*5)
							--Thanks to coworker, now I know that floating point numbers can differ even when their values are the same
							if p == cc then
								love.graphics.rectangle("fill", posx + pixelsX + (x-1)*(pixelSize+pixelOffset), posy + pixelsY + (y-1)*(pixelSize+pixelOffset), pixelSize, pixelSize)
							end
						end
					end
				end
				
				love.graphics.stencil(myStencil, "replace", 1)
				
				love.graphics.setStencilTest("greater", 0)
				love.graphics.draw(minigame.screens.main.on, posx, posy)
				
				love.graphics.setStencilTest()
			elseif i == "main" then -- Main screen not on game
				pixelmode = a/5
				love.graphics.push()
				love.graphics.setColor(1, 1, 1, pixelmode)
				love.graphics.translate(minigame.screens.main.x, minigame.screens.main.y)
				love.graphics.scale(minigame.screens.main.scale, minigame.screens.main.scale)
				
				minigame.setColor(1)
				love.graphics.stencil(_G[state .. "_draw"], "replace", 1)
				minigame.setColor(1)
				love.graphics.pop()
				
				love.graphics.setStencilTest("greater", 0)
				love.graphics.draw(minigame.screens.main.on, minigame.screens.main.x, minigame.screens.main.y)
				love.graphics.setStencilTest()
			else -- Non-game screen
				pixelmode = a/5
				if v.draw then
					love.graphics.setColor(1, 1, 1, pixelmode)
					love.graphics.push()
					love.graphics.translate(v.x, v.y)
					love.graphics.scale(v.scale, v.scale)
					
					minigame.setColor(1)
					love.graphics.stencil(v.draw, "replace", 1)
					minigame.setColor(1)
					love.graphics.setStencilTest("greater", 0)
					love.graphics.draw(v.on, 0, 0)
					love.graphics.setStencilTest()
					
					love.graphics.pop()
				end
			end
		end
	end
end

function love.keypressed(key)
	if (key == "f1") then
		if love._openConsole then
			love._openConsole()
		end
	end
	for i, v in pairs(minigame.buttons) do
		if table.contains(v.keys, key) then
			if v.down == "m" then
				v.down = "km"
			else
				v.down = "k"
				buttonPressed(i)
			end
		end
	end
end

function love.keyreleased(key)
	for i, v in pairs(minigame.buttons) do
		if table.contains(v.keys, key) then
			if v.down == "km" then
				v.down = "m"
			elseif v.down == "k" then
				v.down = false
				buttonReleased(i)
			end
		end
	end
end

function love.mousepressed(x, y, button)
	if button == 1 then button = "l"
	elseif button == 2 then button = "r"
	elseif button == 3 then button = "m" end
	if button == "l" then
		for i, v in pairs(minigame.buttons) do
			local down = false
			if v.shape == "circle" then
				if distance(x, y, v.x, v.y) <= v.radius then
					down = true
				end
			elseif v.shape == "ellipse" then
				--I would (and should) do angle + sin/cos calculations, but I'm lazy okay?
				local xx, yy = x, y
				if v.width > v.height then
					local f = v.height/v.width
					xx = v.x + (x-v.x)*f
				else
					local f = v.width/v.height
					yy = v.y + (y-v.y)*f
				end
				
				if distance(xx, yy, v.x, v.y) <= math.min(v.width, v.height) then
					down = true
				end
			elseif v.shape == "roundrectangle" then
				--Couldn't think of a better, faster way, so I did it corner by corner.
				if x <= v.x+v.radius and y <= v.y+v.radius and distance(x, y, v.x+v.radius, v.y+v.radius) <= v.radius then
					down = true
				elseif x >= v.x+v.width-v.radius and y <= v.y+v.radius and distance(x, y, v.x+v.width-v.radius, v.y+v.radius) <= v.radius then
					down = true
				elseif x <= v.x+v.radius and y >= v.y+v.height-v.radius and distance(x, y, v.x+v.radius, v.y+v.height-v.radius) <= v.radius then
					down = true
				elseif x >= v.x+v.width-v.radius and y >= v.y+v.height-v.radius and distance(x, y, v.x+v.width-v.radius, v.y+v.height-v.radius) <= v.radius then
					down = true
				elseif inside(x, y, 1, 1, v.x, v.y, v.width, v.height) then
					down = true
				end
			elseif table.contains({"arrowup", "arrowleft", "arrowdown", "arrowright"}, v.shape) then
				local n = table.contains({"arrowup", "arrowleft", "arrowdown", "arrowright"}, v.shape)
				
				if inside(x, y, 1, 1, v.x, v.y, v.width, v.height) then
					--Once again, couldn't figure out an easier way
					if n == 1 then
						if x <= v.x+v.width/2 then
							local f = (v.x+v.width/2-x)/v.width/2
							if y-v.y >= v.height*f then
								down = true
							end
						else
							local f = (x-v.x-v.width/2)/v.width/2
							if y-v.y >= v.height*f then
								down = true
							end
						end
					elseif n == 2 then
						if v.y <= v.y+v.height/2 then
							local f = (x-v.x)/v.width
							if y-v.y >= v.height*f then
								down = true
							end
						else
							local f = (v.x-x)/v.width
							if y-v.y >= v.height*f then
								down = true
							end
						end
					elseif n == 3 then
						if x <= v.x+v.width/2 then
							local f = (v.x+v.width/2-x)/v.width/2
							if y-v.y <= v.height*f then
								down = true
							end
						else
							local f = (x-v.x-v.width/2)/v.width/2
							if y-v.y <= v.height*f then
								down = true
							end
						end
					else
						if v.y <= v.y+v.height/2 then
							local f = (x-v.x)/v.width
							if y-v.y < v.height*f then
								down = true
							end
						else
							local f = (v.x-x)/v.width
							if y-v.y < v.height*f then
								down = true
							end
						end
					end
				end
			else --rectangle
				if inside(x, y, 1, 1, v.x-v.width/2, v.y-v.height/2, v.width, v.height) then
					down = true
				end
			end
			
			if down and not v.down then
				v.down = "m"
				buttonPressed(i)
			elseif down and v.down == "k" then
				v.down = "km"
			end
		end
	end
end

function love.mousereleased(x, y, button)
	if button == 1 then button = "l"
	elseif button == 2 then button = "r"
	elseif button == 3 then button = "m" end
	if button == "l" then
		for i, v in pairs(minigame.buttons) do
			if v.down == "km" then
				v.down = "k"
			elseif v.down == "m" then
				v.down = false
				buttonReleased(i)
			end
		end
	end
end

function mkfile()
	local fulltext = ""
	if #games >= 1 then
		for i = 1, #games do
			local text = ""
			if love.filesystem.getInfo("games/" .. games[i] .. ".lua") then
				text = love.filesystem.read("games/" .. games[i] .. ".lua")
			elseif love.filesystem.getInfo("games/" .. games[i] .. ".txt") then
				text = love.filesystem.read("games/" .. games[i] .. ".txt")
			end
			local islocal = "local "
			if i > 1 then islocal = "" end
			fulltext = fulltext .. games[i] .. "_data = {}\r\n" .. islocal .. "self = " .. games[i] .. "_data\r\n"
			
			text = tostring(text)
			
			if i == 1 then
				fulltext = fulltext .. text .. "\r\n"
			else
				fulltext = fulltext .. "\r\n\r\n--==============================--\r\n\r\n" .. text .. "\r\n"
			end
		end
		love.filesystem.write("allgames.lua", fulltext)
		require "allgames"
	end
end

function getImage(file)
	local imgdt = love.image.newImageData(file)
	local points = {}
	local w, h = imgdt:getWidth(), imgdt:getHeight()
	
	for x = 1, 32 do
		points[x] = {}
		for y = 1, 32 do
			points[x][y] = false
		end
	end
	
	for x = 0, w-1 do
		for y = 0, h-1 do
			local r, g, b, a = imgdt:getPixel(x, y)
			if a > 0 then
				points[x+1][y+1] = true
			end
		end
	end
	
	return points
end

function buttonPressed(button)
	table.insert(buttonsDown, button)
	if state == "game" then
		if button == "pause" then
			gamepaused = not gamepaused
		elseif button == "settings" then
			settings_load()
		elseif not gamepaused then
			if _G[currentgame .. "_buttonPressed"] then
				_G[currentgame .. "_buttonPressed"](button)
			end
		end
	else
		_G[state .. "_buttonPressed"](button)
	end
end

function buttonReleased(button)
	if leftMenu == button then
		leftMenu = false
		return
	end
	if state == "game" then
		while table.contains(buttonsDown, button) do
			table.remove(buttonsDown, table.contains(buttonsDown, button))
		end
		if not gamepaused then
			if _G[currentgame .. "_buttonReleased"] then
				_G[currentgame .. "_buttonReleased"](button)
			end
		end
	else
		_G[state .. "_buttonReleased"](button)
	end
end

function loadSet(set, first)
	--[[
		Okay, for once I'm gonna add commentary to my code...
		This function loads minigame "sets" (a.k.a devices)
		While that's mostly a graphical thing, I've included
		ways of actually loading custom settings as well.
	]]
	
	print("___________________\n")
	print("SET LOADING STARTED")
	print("___________________")
	local first = first or false --If first time load, force loading, even if set == minigame.set
	local set = set or minigame.set
	
	--Small function to make my life easier :P
	local function loadImage(s, optional)
		optional = optional or false
		
		_G[s .. "img"] = false
		if love.filesystem.getInfo("sets/" .. set .. "/graphics/" .. s .. ".png") then
			_G[s .. "img"] = love.graphics.newImage("sets/" .. set .. "/graphics/" .. s .. ".png")
			print("["..set.."] loaded custom image " .. s)
		elseif not optional then
			_G[s .. "img"] = love.graphics.newImage("sets/default/graphics/" .. s .. ".png")
			print("["..set.."] loaded default image " .. s)
		else
			print("["..set.."] did not load image " .. s)
		end
	end
	
	print("set = " .. tostring(set))
	print("minigame.set = " .. tostring(minigame.set))
	if set ~= minigame.set or first then
		print("Set: " .. set .. ", minigame.set: " .. minigame.set)
		if first then
			print("First time load")
		end
		loadImage("minigame") --Colorable background, mandatory (otherwise, loads default)
		loadImage("minigameoverlay", true) --Foreground, not mandatory
		loadImage("minigamebackground", true) --Background, not mandatory
		
		--Font-loading, for pixels and stuff
		local function loadFont(n, g)
			local n = n or "font"
			local glyphs = g or "0123456789"
			loadImage(n)
			local fontdata = love.image.newImageData("sets/default/graphics/" .. n .. ".png")
			if love.filesystem.getInfo("sets/" .. set .. "/graphics/" .. n .. ".png") then
				fontdata = love.image.newImageData("sets/" .. set .. "/graphics/" .. n .. ".png")
			end
			_G[n .. "dataquads"] = {}
			for a = 0, string.len(glyphs)-1 do
				local t = {}
				for x = 1, math.floor(fontdata:getWidth()/string.len(glyphs)) do
					t[x] = {}
					for y = 1, fontdata:getHeight() do
						local r, g, b, a = fontdata:getPixel(x-1 + a*6, y-1)
						if a > .2 then
							t[x][y] = 1
						else
							t[x][y] = 0
						end
					end
				end
				_G[n .. "dataquads"][string.sub(glyphs, a+1, a+1)] = t
			end
		end
		
		--Fonts, not mandatory but loads default otherwise
		loadFont("font", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789.!?:-#&()[]{}<>")
		loadFont("bignumbers")
		loadFont("smallnumbers")
		
		minigame.screens = {}
		
		local screens = love.filesystem.getDirectoryItems("sets/default/screens")
		local currentset = "default"
		
		--Loads sets screens. A sets can have as many screens as it can, but MUST have a main screen
		if love.filesystem.getInfo("sets/" .. set .. "/screens") and love.filesystem.getInfo("sets/" .. set .. "/screens/main.png") then
			screens = love.filesystem.getDirectoryItems("sets/" .. set .. "/screens")
			currentset = set
			print("["..set.."] has screens")
		end
		
		--First we set all the "off" screens
		for i, v in ipairs(screens) do
			if string.sub(v, -4, -1) == ".png" and string.sub(v, -7, -5) ~= "_on" then
				if set == currentset then
					print("["..set.."] screen \"" .. string.sub(v, 1, -5) .. "\" has 'off' sprite")
				end
				minigame.screens[string.sub(v, 1, -5)] = {}
				minigame.screens[string.sub(v, 1, -5)]["off"] = love.graphics.newImage("sets/" .. currentset .. "/screens/" .. v)
			end
		end
		
		--Then we check if all the "on" screens have a corresponding "off"
		for i, v in ipairs(screens) do
			if string.sub(v, -7, -5) == "_on" and minigame.screens[string.sub(v, 1, -8)] then
				if set == currentset then
					print("["..set.."] screen \"" .. string.sub(v, 1, -8) .. "\" has 'on' sprite")
				end
				minigame.screens[string.sub(v, 1, -8)]["on"] = love.graphics.newImage("sets/" .. currentset .. "/screens/" .. v)
			end
			
			--Simultaneously, check for custom drawing functions for custom screens
			local form
			if string.sub(v, -4, -1) == ".txt" then
				form = ".txt"
			elseif string.sub(v, -4, -1) == ".lua" then
				form = ".lua"
			end
			if form then
				local n = string.sub(v, 1, -5)
				print("FORMYA " .. n)
				if minigame.screens[n] then
					print("SCREENYA")
					local chunk = love.filesystem.load("sets/" .. currentset .. "/screens/" .. n .. form)
					local ok, err = pcall(chunk)
					if ok then
						minigame.screens[n].draw = chunk
						if set == currentset then
							print("["..set.."] screen \"" .. n .. "\" has custom drawing function")
						end
					elseif set == currentset then
						print("["..set.."] screen \"" .. n .. "\" had an error with custom drawing function")
						print(err)
					end
				end
			end
		end
		
		--Last, but not least, we check if all "off" screens have "on". Otherwise, render a fake one just because
		for i, v in pairs(minigame.screens) do
			if not v.on then
				print("["..set.."] screen \"" .. i .. "\" did not have 'on' sprite. Making one")
				local fake = love.image.newImageData("sets/" .. currentset .. "/screens/" .. i .. ".png")
				fake:mapPixel(
					function(x, y, r, g, b, a)
						return r*.2, g*.2, b*.2, a --Darker version of the screen
					end
				)
			end
			
			--Default position for safety
			v.x = 100
			v.y = 100
			
			v.scale = 1
			v.offsetX = 1
			v.offsetY = 1
			v.pixelSize = 5
			v.pixelOffset = 1
		end
		
		--Default pixel sizes
		
		--Load screen settings, to define their position
		local s = love.filesystem.read("sets/default/screens.txt")
		if love.filesystem.getInfo("sets/" .. set .. "/screens.txt") then
			s = love.filesystem.read("sets/" .. set .. "/screens.txt")
		end
		s = string.gsub(s, "\r\n", "\n")
		s = string.gsub(s, " ", "")
		local s1 = s:split("\n")
		for i, v in ipairs(s1) do
			local s2 = v:split("=")
			if minigame.screens[ s2[1] ] then
				local s3 = s2[2]:split(",")
				if #s3 >= 2 then
					if tonumber(s3[1]) then
						minigame.screens[ s2[1] ].x = math.floor( math.abs( tonumber(s3[1]) ) )
						if set == currentset then
							print("["..set.."] screen \"" .. s2[1] .. "\" custom X position set to " .. s3[1])
						end
					end
					if tonumber(s3[2]) then
						minigame.screens[ s2[1] ].y = math.floor( math.abs( tonumber(s3[2]) ) )
						if set == currentset then
							print("["..set.."] screen \"" .. s2[1] .. "\" custom Y position set to " .. s3[2])
						end
					end
					if (#s3 >= 3 and tonumber(s3[3])) then
						minigame.screens[ s2[1] ].scale = tonumber(s3[3])
					end
					if (#s3 >= 4 and tonumber(s3[4])) then
						minigame.screens[ s2[1] ].offsetX = tonumber(s3[4])
					end
					if (#s3 >= 5 and tonumber(s3[5])) then
						minigame.screens[ s2[1] ].offsetY = tonumber(s3[5])
					end
					if (#s3 >= 6 and tonumber(s3[6])) then
						minigame.screens[ s2[1] ].pixelSize = tonumber(s3[6])
					end
					if (#s3 >= 7 and tonumber(s3[7])) then
						minigame.screens[ s2[1] ].pixelOffset = tonumber(s3[7])
					end
				end
			end
		end
		
		
		--Next up, we're gonna load the button images and settings. Some buttons are mandatory, but we can have custom ones as well.
		minigame.buttons = {}
		
		currentset = "default"
		
		--Run it twice, so that I'm 100% sure that AT LEAST the default buttons will load
		for a = 1, 2 do
			local s = ""
			if love.filesystem.getInfo("sets/" .. currentset .. "/buttons.txt") then
				s = love.filesystem.read("sets/" .. currentset .. "/buttons.txt")
				if set == currentset then
					print("["..set.."] has custom button settings")
				end
			end
			
			--Go through the buttons settings file. Most of the code is "debugging", that's why it's huge
			s = string.gsub(s, "\r\n", "\n")
			s = string.gsub(s, " ", "")
			local s1 = s:split("\n")
			if #s1 >= 1 then
				for i, v in ipairs(s1) do
					local s2 = v:split("=")
					minigame.buttons[ s2[1] ] = {}
					
					local s3 = s2[2]:split(",")
					
					--circle, x, y, r, keys...
					--roundrectangle, x, y, w, h, r, keys...
					--(other shapes), x, y, w, h, keys...
					if table.contains(button_shapes, s3[1]) then
						minigame.buttons[ s2[1] ].shape = s3[1]
						if set == currentset then
							print("["..set.."] button \"" .. s2[1] .. "\" has shape " .. s3[1])
						end
					end
					table.remove(s3, 1)
					
					if tonumber(s3[1]) then
						minigame.buttons[ s2[1] ].x = math.abs( math.floor( tonumber(s3[1] ) ) )
						if set == currentset then
							print("["..set.."] button \"" .. s2[1] .. "\" has position X " .. s3[1])
						end
					end
					table.remove(s3, 1)
					
					if tonumber(s3[1]) then
						minigame.buttons[ s2[1] ].y = math.abs( math.floor( tonumber(s3[1] ) ) )
						if set == currentset then
							print("["..set.."] button \"" .. s2[1] .. "\" has position Y " .. s3[1])
						end
					end
					table.remove(s3, 1)
					
					if minigame.buttons[ s2[1] ].shape == "circle" then --radius
						if tonumber(s3[1]) then
							minigame.buttons[ s2[1] ].radius = math.abs( math.floor( tonumber(s3[1] ) ) )
							if set == currentset then
								print("["..set.."] button \"" .. s2[1] .. "\" has radius " .. s3[1])
							end
						end
						table.remove(s3, 1)
					else --width & height
						if tonumber(s3[1]) then
							minigame.buttons[ s2[1] ].width = math.abs( math.floor( tonumber(s3[1] ) ) )
							if set == currentset then
								print("["..set.."] button \"" .. s2[1] .. "\" has width " .. s3[1])
							end
						end
						table.remove(s3, 1)
						if tonumber(s3[1]) then
							minigame.buttons[ s2[1] ].height = math.abs( math.floor( tonumber(s3[1] ) ) )
							if set == currentset then
								print("["..set.."] button \"" .. s2[1] .. "\" has height " .. s3[1])
							end
						end
						table.remove(s3, 1)
						
						if minigame.buttons[ s2[1] ].shape == "roundrectangle" then --radius
							if tonumber(s3[1]) then
								minigame.buttons[ s2[1] ].radius = math.abs( math.floor( tonumber(s3[1] ) ) )
								if set == currentset then
									print("["..set.."] button \"" .. s2[1] .. "\" has radius " .. s3[1])
								end
							end
							table.remove(s3, 1)
						end
					end
					
					minigame.buttons[ s2[1] ].keys = {unpack(s3)}
					minigame.buttons[ s2[1] ].down = false
					
					if set == currentset then
						print("["..set.."] button \"" .. s2[1] .. "\" has keys: " .. table.concat(minigame.buttons[ s2[1] ].keys, ", "))
					end
				end
			end
			
			--Loading images now. First we check each button image and associate with existing settings
			local t = love.filesystem.getDirectoryItems("sets/" .. currentset .. "/buttons")
			for i, v in ipairs(t) do
				if string.sub(v, -4, -1) == ".png" and string.sub(v, -9, -5) ~= "_down" then
					local n = string.sub(v, 1, -5)
					if minigame.buttons[n] then
						minigame.buttons[n].image = love.graphics.newImage("sets/" .. currentset .. "/buttons/" .. v)
						if set == currentset then
							print("["..set.."] button \"" .. n .. "\" has custom up image")
						end
					end
				end
			end
			
			--Then we check existing "_down" images and associate with existing images
			for i, v in ipairs(t) do
				if string.sub(v, -4, -1) == ".png" and string.sub(v, -9, -5) == "_down" then
					local n = string.sub(v, 1, -10)
					if minigame.buttons[n] and minigame.buttons[n].image then
						minigame.buttons[n].imagedown = love.graphics.newImage("sets/" .. currentset .. "/buttons/" .. v)
						if set == currentset then
							print("["..set.."] button \"" .. n .. "\" has custom down image")
						end
					end
				end
			end
			
			--Then, much like the set thing, we create fake "_down" images if none exists.
			--Just an FYI: in case a button has no images, it'll be drawn as coded geometry ;)
			for i, v in pairs(minigame.buttons) do
				if v.image and not v.imagedown then
					print("OKE DOKE")
					local fake
					if love.filesystem.getInfo("sets/" .. set .. "/buttons/" .. i .. ".png") then
						fake = love.image.newImageData("sets/" .. set .. "/buttons/" .. i .. ".png")
					else
						fake = love.image.newImageData("sets/default/buttons/" .. i .. ".png")
					end
					if set == currentset then
						print("["..set.."] button \"" .. i .. "\" has no custom down image. Making one")
					end
					
					--Fake images will be image with inverted colors and inverted hue (a.k.a illusion of "in" more than "out")
					fake:mapPixel(
						function(x, y, r, g, b, a)
							r = 1-r
							g = 1-g
							b = 1-b
							
							color.setColor(r, g, b)
							color.selectionangle = color.selectionangle + math.pi
							if color.selectionangle >= math.pi*2 then
								color.selectionangle = color.selectionangle - math.pi*2
							end
							r, g, b = color.getColor()
							
							return r*.25, g*.25, b*.25, a
						end
					)
					
					v.imagedown = love.graphics.newImage(fake)
				end
			end
			
			if love.filesystem.getInfo("sets/" .. set .. "/buttons") then
				currentset = set
			end
		end
		
		minigame.set = set
		
		print("SET = " .. minigame.set)
		
		local ww, hh = minigameimg:getWidth(), minigameimg:getHeight()
		if ww ~= love.window.getWidth() or hh ~= love.window.getHeight() then
			love.window.setMode(ww, hh)
			print("Window resized successfully")
		end
	end
	
	print("________________________\n")
	print("Set loaded successfully!")
	print("________________________")
end

function love.focus(f)
	if not f and state == "game" then
		gamepaused = true
	end
end

function love.quit()
	minigame.saveSettings()
	minigame.saveGameSettings()
end


--[[
	Progress map:
	
	■ - Documented
	□ - Not documented/documentation unnecessary
	
	√ - Fully implemented
	≈ - Partially implemented or in need of optimization
	≠ - Not implemented/implemented in a rough way
]]

-------------------------
-- Utilities functions --
-------------------------

-- ■ √
function string:split(delimiter) --Not by me
	local result = {}
	local from	= 1
	local delim_from, delim_to = string.find( self, delimiter, from	 )
	while delim_from do
		table.insert( result, string.sub( self, from , delim_from-1 ) )
		from = delim_to + 1
		delim_from, delim_to = string.find( self, delimiter, from  )
	end
	table.insert( result, string.sub( self, from  ) )
	return result
end

-- ■ √
function distance(x1, y1, x2, y2)
	local w, h
	if x1 == x2 then w = 0 end
	if y1 == y2 then h = 0 end
	if x1 > x2 then w = x1 - x2 end
	if x1 < x2 then w = x2 - x1 end
	if y1 > y2 then h = y1 - y2 end
	if y1 < y2 then h = y2 - y1 end
	return math.sqrt(w*w + h*h)
end

-- ■ √
function table.contains(t, e)
	for i, v in pairs(t) do
		if v == e then
			return i
		else
			if type(v) == "table" and type(e) == "table" then
				local can = true
				for a, b in pairs(v) do
					if e[a] ~= v[a] then
						can = false
						break
					end
				end
				if can then return i end
			end
		end
	end
	return false
end

-- ■ √
function table.copy(t)
	local ret = {}
	for i, v in pairs(t) do
		if type(v) == "table" then
			ret[i] = table.copy(v)
		else
			ret[i] = v
		end
	end
	return ret
end

-- ■ √
function round(n, d)
	local d = d or 2
	return math.floor(n*10^d)/10^d
end

-- ■ √
function addZeros(n, l)
	n = tostring(n)
	l = l or 2
	while string.len(n) < l do
		n = "0" .. n
	end
	return n
end

-- ■ √
function inside(x1, y1, w1, h1, x2, y2, w2, h2)
	return x1 >= x2 and x1+w1 <= x2+w2 and y1 >= y2 and y1+h1 <= y2+h2
end

------------------------
-- Minigame functions --
------------------------

-- ■ √
function minigame.isPaused()
	return gamepaused
end

-- ■ √
function minigame.addScore(n)
	n = math.max(0, n)
	n = math.min(1000000, n)
	n = math.floor(n/100)*100 --only multiples of 100
	gamescore = gamescore + n
	gamescore = math.min(gamescore, 1000000)
	updateHighscore()
end

-- ■ √
function minigame.removeScore(n)
	n = math.max(0, n)
	n = math.min(1000000, n)
	n = math.floor(n/100)*100 --only multiples of 100
	gamescore = gamescore - n
	gamescore = math.max(gamescore, 0)
	updateHighscore()
end

-- ■ √
function minigame.setScore(n)
	n = math.max(0, n)
	n = math.min(1000000, n)
	n = math.floor(n/100)*100 --only multiples of 100
	gamescore = n
	gamescore = math.max(gamescore, 0)
	gamescore = math.min(gamescore, 1000000)
	updateHighscore()
end

-- ■ √
function minigame.getScore()
	return gamescore
end

-- ■ √
function minigame.getHighscore()
	return gamehighscore
end

-- □ √
function updateHighscore()
	if gamehighscore < gamescore then
		gamehighscore = gamescore
	end
	if gamehighscores[currentgame] then
		if gamehighscore > gamehighscores[currentgame] then
			gamehighscores[currentgame] = gamehighscore
		else
			gamehighscore = gamehighscores[currentgame]
		end
	else
		gamehighscores[currentgame] = gamehighscore
	end
end

-- ■ √
function minigame.resetLevel()
	_G[currentgame .. "_load"](gamelevel)
end

-- ■ √
function minigame.levelUp(ret)
	gamelevel = gamelevel+1
	
	if _G[currentgame .. "_data"].maxLevel and gamelevel > _G[currentgame .. "_data"].maxLevel then
		-- Gameover
		gamelevel = _G[currentgame .. "_data"].maxLevel
		
		if _G[currentgame .. "_gameOver"] then
			_G[currentgame .. "_gameOver"]()
		else
			minigame.gameOver()
		end
		return -- I frankly don't know why I didn't handle it like this before
	end
	
	local ok, err = pcall(_G[currentgame .. "_load"], gamelevel)
	
	if ok then
		_G[currentgame .. "_load"](gamelevel)
		if _G[currentgame .. "_currentLevel"] < gamelevel then
			_G[currentgame .. "_currentLevel"] = gamelevel
		end
	elseif ret then --Cheating the code just for the menu
		gamelevel = gamelevel-1
	else --Game doesn't go up to this level
		print(err)
		--So we either call a "lastLevel" sort of function...
		if _G[currentgame .. "_maxLevel"] < gamelevel then
			gamelevel = gamelevel-1
		else -- ...or we gameOver the player. I was going to go back to level 1, but not all games are non-linear.
			if _G[currentgame .. "_gameOver"] then
				_G[currentgame .. "_gameOver"]()
			else
				minigame.gameOver()
			end
		end
	end
end

-- ■ √
function minigame.getLevel()
	return gamelevel
end

-- ■ √
function minigame.buttonDown(button)
	return (minigame.buttons[button].down ~= false and true) or false
end

-- ■ √
function minigame.lastButton(i)
	if i then
		if #buttonsDown >= 1 and buttonsDown[#buttonsDown] == i then
			return true
		end
		return false
	elseif #buttonsDown >= 1 then
		return buttonsDown[#buttonsDown]
	else
		return false
	end
end

-- ■ √
function minigame.setLives(n)
	if type(n) == "number" then
		n = math.floor(math.max(0, n))
		gamelives = n
	elseif n == false then
		gamelives = false
	end
	
	if gamelives == 0 then
		minigame.gameOver()
	end
end

-- ■ √
function minigame.addLives(n)
	if type(n) == "number" and gamelives then
		n = math.floor(math.max(0, n))
		gamelives = gamelives+n
	end
end

-- ■ √
function minigame.getLives()
	return gamelives
end

-- ■ √
function minigame.removeLives(n)
	if type(n) == "number" and gamelives then
		n = math.floor(math.max(0, n))
		gamelives = math.max(0, gamelives-n)
	end
	
	if gamelives == 0 then
		if _G[currentgame .. "_gameOver"] then
			_G[currentgame .. "_gameOver"]()
		else
			minigame.gameOver()
		end
	end
end

-- ■ √
function minigame.gameOver()
	minigame.saveSettings() --I guess I have to
	state = "menu"
	menu_load()
end

-- ■ √
function minigame.timeUp()
	--I'm not doing gameover or removing lives cause I dunno what the game maker does with timer.
	--So I'm just reloading the level. His fault, not mine...
	if minigame.getLives() > 1 then
		minigame.removeLives(1)
		loadGame(gamelevel)
	else
		if _G[currentgame .. "_gameOver"] then
			_G[currentgame .. "_gameOver"]()
		else
			minigame.gameOver()
		end
	end
end

-- ■ √
function minigame.setTime(n)
	if n then
		if tonumber(n) then --applies to string numbers or just number numbers
			gametime = math.max( math.floor(tonumber(n)), 0 )
		elseif type(n) == "string" then --We assume there's a collon, like MM:SS
			local s = n:split(":")
			if #s == 2 then
				if tonumber(s[1]) and tonumber(s[2]) then
					local mm = math.max( math.floor(tonumber(s[1])), 0 )
					local ss = math.max( math.floor(tonumber(s[2])), 0 )
					
					mm = math.min(mm, 59)
					ss = math.min(ss, 59)
					
					gametime = mm*60 + ss
				end
			end
		end
	
		gametime = math.min(gametime, 59*60+59)
		gametime = math.max(gametime, 0)
	else
		gametime = false
	end
end

-- ■ √
function minigame.getTime(mode)
	local mode = mode or "string"
	if mode == "string" and gametime == false then
		return "--:--"
	elseif gametime == false then
		return false
	elseif mode == "string" then
		local mm = math.floor(gametime/60)
		local ss = math.floor(gametime - mm*60)
		
		return addZero(mm) .. ":" .. addZero(ss)
	else
		return math.floor(gametime)
	end
end

-- ■ √
function minigame.addTime(n)
	--I'm super lazy and don't want to rewrite some code, so watch how a lazy programmer uses his brain to save up time:
	local oldgametime = gametime
	minigame.setTime(n)
	if gametime > 60*10 then --No more than 10 minutes added
		minigame.setTime(oldgametime + 60*10)
	else
		minigame.setTime(oldgametime + gametime)
	end
	--See? I used my setTime function cause it deals with strings, trimming, etc.
end

-- ■ √
function minigame.removeTime(n)
	--Time to be lazy again!
	local oldgametime = gametime
	minigame.setTime(n)
	if gametime > 60*10 then --No more than 10 minutes removed
		minigame.setTime(oldgametime - 60*10)
	else
		minigame.setTime(oldgametime - gametime)
	end
end

-- ■ √
function minigame.setSpeed(n)
	if type(n) == "number" then
		n = math.max(n, 0)
		if n >= 1 then
			n = math.ceil(n)
		else
			n = math.floor(n*5)/5 --Multiples of .2
		end
		n = math.max(.4, n)
		n = math.min(10, n)
		gamespeed = n
	end
end

-- ■ √
function minigame.addSpeed(n)
	if type(n) == "number" then
		n = math.max(n, 0)
		n = math.max(.2, n)
		n = math.min(10, n)
		
		if gamespeed+n >= 1 then
			minigame.setSpeed( math.ceil(gamespeed+n) )
		else
			minigame.setSpeed(gamespeed+n)
		end
	end
end

-- ■ √
function minigame.removeSpeed(n)
	if type(n) == "number" then
		n = math.max(n, 0)
		n = math.max(.2, n)
		n = math.min(10, n)
		
		if gamespeed-n >= 1 then
			minigame.setSpeed( math.floor(gamespeed-n) )
		else
			minigame.setSpeed(gamespeed-n)
		end
	end
end

-- ■ √
function minigame.getSpeed()
	return gamespeed
end

-- ■ √
function minigame.setProgress(n)
	if type(n) == "number" then
		n = math.max(n, 0)
		n = math.min(1, n)
		
		--Progress is between 0 and 1
		
		gameprogress = n
	end
end

-- ■ √
function minigame.getProgress()
	return gameprogress
end

-- □ √
function regularprint(txt, x, y, scale)
	local txt = tostring(txt)
	local scale = scale or 1
	
	local function drawChar(c, x, y)
		for i, v in ipairs(c) do
			love.graphics.rectangle("fill", x+v[1]*scale, y+v[2]*scale, scale, scale)
		end
	end
	
	for i = 1, string.len(txt) do
		local px = x+(i-1)*6*scale
		local c = string.sub(txt, i, i)
		if regularfontquads[c] then
			drawChar(regularfontquads[c], px, y)
		elseif c == "|" then
			x = -i*6+1
			y = y+11
		elseif c ~= " " then
			drawChar(regularfontquads["?"], px, y)
		end
	end
end

--------------
-- SETTINGS --
--------------
-- □ √
function minigame.loadSettings()
	collectgarbage()
	print("MINIGAME.LOADSETTINGS")
	
	-------------------
	-- Load Settings --
	-------------------
	if love.filesystem.getInfo("savedata.lua") then
		local chunk = love.filesystem.load("savedata.lua")
		local ok, vartable = pcall(chunk)
		if ok then
			chunk()
			if lastgame and not currentgame then
				currentgame = lastgame
			end
		else
			print("Error while loading settings")
		end
	end
	
	if not currentgame then
		currentgame = games[1]
	end
	
	---------------------
	-- Load Highscores --
	---------------------
	if love.filesystem.getInfo("highscore.lua") then
		local txt = love.filesystem.read("highscore.lua")
		txt = string.gsub(txt, "\r\n", "\n")
		local s = txt:split("\n")
		if #s >= 1 then
			for i = 1, #s do
				local s1 = s[i]:split(" = ")
				if #s1 == 2 then
					if table.contains(games, s1[1]) then
						if tonumber(s1[2]) then
							gamehighscores[ s1[1] ] = math.abs( math.floor(tonumber(s1[2])) )
						end
					end
				end
			end
		end
	end
end

-- □ √
function minigame.saveSettings()
	print("MINIGAME.SAVESETTINGS")
	-------------------
	-- Save Settings --
	-------------------
	local t = "minigame.skincolor = {" .. minigame.skincolor[1] .. ", " .. minigame.skincolor[2] .. ", " .. minigame.skincolor[3] .. "}"
	t = t .. "\r\nminigame.set = \"" .. minigame.set .. "\""
	t = t .. "\r\ngamespeed = " .. gamespeed
	t = t .. "\r\ngamelevel = " .. gamelevel
	if seed == false then seed = "random" end
	t = t .. "\r\nseed = \"" .. seed .. "\""
	t = t .. "\r\nlastgame = \"" .. currentgame .. "\""
	
	love.filesystem.write("savedata.lua", t)
	
	---------------------
	-- Save Highscores --
	---------------------
	local t = ""
	local s = ""
	for i, v in pairs(gamehighscores) do
		t = t .. s .. i .. " = " .. v
		s = "\r\n"
	end
	
	love.filesystem.write("highscore.lua", t)
end

-- □ √
function minigame.loadGameSettings(game)
	local game = game or currentgame
	print("MINIGAME.LOADGAMESETTINGS(" .. game .. ")")
	
	if love.filesystem.getInfo("gamedata/" .. game .. "_data.lua") then
		love.filesystem.load("gamedata/" .. game .. "_data.lua")()
	end
	
	if not _G[game .. "_data"] then
		_G[game .. "_data"] = {}
	end
	
	if not _G[game .. "_data"].maxLevel then
		if not _G[game .. "_maxLevel"] then
			--Stress-test the game up to level 100 to see if it works
			--Now it only does this the very first time the game is loaded :D
			local maxLevel = 1
			for j = 1, 100 do
				local ok, err = pcall(_G[game .. "_load"], j)
				if ok then
					maxLevel = j
				else
					break
				end
			end
			_G[game .. "_maxLevel"] = maxLevel
		end
	else
		_G[game .. "_maxLevel"] = _G[game .. "_data"].maxLevel
	end
	
	
	if not _G[game .. "_currentLevel"] then
		_G[game .. "_currentLevel"] = 1
	end
	minigame.saveGameSettings(game) --This way, stress-testing doesn't happen all the time
end

-- □ √
function minigame.saveGameSettings(game)
	local game = game or currentgame
	print("MINIGAME.SAVEGAMESETTINGS(" .. game .. ")")
	love.filesystem.createDirectory("gamedata")
	local t = ""
	if _G[game .. "_data"].settings then
		for j, w in pairs(_G[game .. "_data"].settings) do
			t = t .. game .. "_data." .. w[2] .. " = " .. datatostring(_G[game .. "_data"][w[2]]) .. "\r\n"
		end
	end
	t = t .. "\r\n" .. game .. "_currentLevel = " .. datatostring(_G[game .. "_currentLevel"])
	t = t .. "\r\n" .. game .. "_maxLevel = " .. datatostring(_G[game .. "_maxLevel"])
	
	love.filesystem.write("gamedata/" .. game .. "_data.lua", t)
end

-- □ √
function minigame.loadGame(game)
	minigame.setLives(false)
	minigame.setTime(false)
	local game = game or currentgame
	if love.filesystem.getInfo("games/" .. game .. ".lua") then
		if not _G[game .. "_data"] then
			_G[game .. "_data"] = {}
		end
		self = _G[game .. "_data"]
		love.filesystem.load("games/" .. game .. ".lua")()
	end
end

-- □ √
function datatostring(data)
	local t = ""
	if type(data) == "string" then
		t = "\"" .. data .. "\""
	elseif type(data) == "table" then
		t = "{"
		local e = {}
		local c = {}
		local isArray = true
		for i, v in pairs(data) do
			if type(i) ~= "number" then
				isArray = false
			end
			table.insert(e, i)
			table.insert(c, datatostring(v))
		end
		if isArray then
			for i, v in ipairs(data) do
				t = t .. c[i] .. ", "
			end
			t = string.sub(t, 1, -3) .. "}"
		else
			for i, v in ipairs(e) do
				t = t .. "\r\n" .. datatostring(v) .. " = " .. c[i] .. ", "
			end
			t = string.sub(t, 1, -3) .. "\r\n}"
		end
	else
		t = tostring(data)
	end
	return t
end

-- □ √
function minigame.clearScreen()
	for x = 1, pixelsH do
		for y = 1, pixelsV do
			pixel[x][y] = 0
		end
	end
end

-- □ √
function minigame.drawScreen()
	for x = 1, pixelsH do
		for y = 1, pixelsV do
			love.graphics.setColor(1, 1, 1, (pixel[x][y]/5))
			love.graphics.rectangle("fill", (x-1)*5+x, (y-1)*5+y, 5, 5)
		end
	end
end

--[[
	Okay, I admit: I stole this code from my Pattern Generator.
	But I couldn't figure out a simpler way of changing a pixel's
	hue when generating a fake image for pressed buttons :(
]]
color = {}
color.selectionangle = 0

function color.getColor(dat)
	local dat = dat or "all"
	local r, g, b = 0, 0, 0
	local aa = math.pi/2
	
			if dat == "hue" or dat == "all" or (string.sub(dat, 1, 1) == "!" and dat ~= "!hue") then
	--=====--
	-- HUE --
	--=====--
	if color.selectionangle >= aa/3 and color.selectionangle <= math.pi-aa/3 then
		r = 1
		if color.selectionangle >= aa then
			g = color.selectionangle-aa
			g = g/(aa/3*2)
		else
			b = color.selectionangle - aa/3
			b = (1-b/(aa/3*2))
		end
	elseif color.selectionangle > math.pi-aa/3 and color.selectionangle <= 3*aa then
		g = 1
		if color.selectionangle <= math.pi+aa/3 then
			r = color.selectionangle - (aa+aa/3*2)
			r = (1-r/(aa/3*2))
		else
			b = color.selectionangle - (math.pi+aa/3)
			b = b/(aa/3*2)
		end
	else
		b = 1
		if color.selectionangle <= math.pi+aa+aa/3*2 and color.selectionangle >= math.pi+aa then
			g = color.selectionangle - (math.pi+aa)
			g = (1-g/(aa/3*2))
		else
			r = color.selectionangle-aa/3
			if r < 0 then r = r + math.pi*2 end
			r = r - (math.pi+aa+aa/3)
			r = r/(aa/3*2)
		end
	end
	
			end
	
			if dat == "sat" or dat == "all" or (string.sub(dat, 1, 1) == "!" and dat ~= "!sat") then
	--============--
	-- SATURATION --
	--============--
	local sat = math.max(r, g, b)
		r = r + (sat-r)*color.saturangle
		g = g + (sat-g)*color.saturangle
		b = b + (sat-b)*color.saturangle
			end
	
			if dat == "bri" or dat == "all" or (string.sub(dat, 1, 1) == "!" and dat ~= "!bri") then
	--============--
	-- BRIGHTNESS --
	--============--
	
		r = r * (1-color.brightangle)
		g = g * (1-color.brightangle)
		b = b * (1-color.brightangle)
			end
	
	if r >= 1 then r = 1 elseif r <= 0 then r = 0 end
	if g >= 1 then g = 1 elseif g <= 0 then g = 0 end
	if b >= 1 then b = 1 elseif b <= 0 then b = 0 end
	return r, g, b, 1
end

function color.setColor(...)
	local args = {...}
	local c = {}
	if type(args[1]) == "table" then
		c = {unpack(args[1])}
	elseif type(args[1]) == "string" then
		c = self:hexrgb(args[1])
	else
		c = {unpack(args)}
	end
	
	while #c > 3 do
		table.remove(c)
	end
	
	
	color.saturangle =  math.min(unpack(c))/math.max(unpack(c))
	if math.max(unpack(c)) == 0 then
		color.saturangle = 0
	end
	color.brightangle = 1-math.max(unpack(c))
	
	local r, g, b = unpack(c)
	
	r, g, b = r-math.min(r, g, b), g-math.min(r, g, b), b-math.min(r, g, b)
	if math.max(r, g, b) > 0 then
		r, g, b = r/math.max(r, g, b), g/math.max(r, g, b), b/math.max(r, g, b)
	end
	
	if r <= 0 then r = 0 end
	if g <= 0 then g = 0 end
	if b <= 0 then b = 0 end
	
	--=====--
	-- HUE --
	--=====--
	local aa = math.pi/2
	if r == 1 then
		color.selectionangle = aa
		if g > 0 then
			color.selectionangle = color.selectionangle + g*(aa/3*2)
		else
			color.selectionangle = color.selectionangle - b*(aa/3*2)
		end
	elseif g == 1 then
		color.selectionangle = math.pi+aa/3
		if b > 0 then
			color.selectionangle = color.selectionangle + b*(aa/3*2)
		else
			color.selectionangle = color.selectionangle - r*(aa/3*2)
		end
	elseif b == 1 then
		color.selectionangle = math.pi*2-aa/3
		if r > 0 then
			color.selectionangle = color.selectionangle + r*(aa/3*2)
		else
			color.selectionangle = color.selectionangle - g*(aa/3*2)
		end
	end
	
	if color.selectionangle < 0 then
		color.selectionangle = color.selectionangle + math.pi*2
	elseif color.selectionangle >= math.pi*2 then
		color.selectionangle = color.selectionangle - math.pi*2
	end
end

-- Not by me, acquired from user Kornel Kisielewicz
-- http://stackoverflow.com/questions/2049582/how-to-determine-a-point-in-a-2d-triangle
function detectCollision(px, py, v1x, v1y, v2x, v2y, v3x, v3y)
	local function sign (p1x, p1y, p2x, p2y, p3x, p3y)
		return (p1x - p3x) * (p2y - p3y) - (p2x - p3x) * (p1y - p3y)
	end
	
	local b1 = sign(px, py, v1x, v1y, v2x, v2y) < 0
	local b2 = sign(px, py, v2x, v2y, v3x, v3y) < 0
	local b3 = sign(px, py, v3x, v3y, v1x, v1y) < 0
	
	return ((b1 == b2) and (b2 == b3))
end

function love.window.getWidth()
	local ww, hh = love.window.getMode()
	return ww
end

function love.window.getHeight()
	local ww, hh = love.window.getMode()
	return hh
end
