menut = {}
local self = menut
function menu_load()
	gamepaused = false
	self.selectedgame = 1
	
	if currentgame then
		self.selectedgame = table.contains(games, currentgame) or 1
	else
		currentgame = games[1]
	end
	
	function self.update()
		minigame.loadGame(currentgame)
		minigame.loadGameSettings(currentgame)
		local level_toload = gamelevel
		if _G[currentgame .. "_data"].maxLevel then
			level_toload = math.min(gamelevel, _G[currentgame .. "_data"].maxLevel)
		end
		if _G[currentgame .. "_load"] then
			_G[currentgame .. "_load"](level_toload)
		end
		
		gamehighscore = gamehighscores[currentgame] or 0
		self.gamename = _G[currentgame .. "_data"].name or currentgame
		self.gameicon = _G[currentgame .. "_data"].icon or false
		self.gameauthor = _G[currentgame .. "_data"].author
		if type(self.gameauthor) ~= "string" then
			self.gameauthor = "Unknown"
		end
		if not self.gameicon and love.filesystem.getInfo("games/" .. currentgame .. "_icon.png") then
			self.gameicon = "games/" .. currentgame .. "_icon.png"
		end
		
		if self.gameicon then
			if type(self.gameicon) == "string" then
				local add = {"", "games/", "games/" .. currentgame .. "/", "games/" .. currentgame .. "/graphics/"}
				local canbreak = false
				for i, v in ipairs(add) do
					for j = 1, 2 do
						local a = ""
						if j == 2 then a = ".png" end
						if love.filesystem.getInfo(v .. self.gameicon .. a) then
							self.gameicon = love.image.newImageData(v .. self.gameicon .. a)
							canbreak = true
						end
						
						if canbreak then break end
					end
					if canbreak then break end
				end
			end
			
			if type(self.gameicon) == "userdata" and self.gameicon:type() == "ImageData" then
				self.gameicon = imageToData(self.gameicon)
			elseif type(self.gameicon) ~= "table" then
				self.gameicon = false
			end
		end
		
		if not self.gameicon then
			minigame.clearScreen()
			--REMEMBER KIDS: Always clear your screen before pre-rendering images.
			--Just start the simulator with a game without icon and without this call, you'll see why I need this.
			
			pixelmode = false
			_G[currentgame .. "_draw"]()
			self.gameicon = {}
			
			local function average(x, y)
				local p1, p2, p3, p4 = 0, 0, 0, 0
				if pixel[x] and pixel[x][y] then
					p1 = pixel[x][y]
				end
				if pixel[x] and pixel[x][y+1] then
					p2 = pixel[x][y+1]
				end
				if pixel[x+1] and pixel[x+1][y] then
					p3 = pixel[x+1][y]
				end
				if pixel[x+1] and pixel[x+1][y+1] then
					p4 = pixel[x+1][y+1]
				end
				
				return minigame.fixedColor((p1+p2+p3+p4)/2)
			end
			
			for x = 1, 16 do
				self.gameicon[x] = {}
				for y = 1, 16 do
					self.gameicon[x][y] = average(x*2-1, y*2-1) or 0
				end
			end
			
			_G[currentgame .. "_data"].icon = {unpack(self.gameicon)}
		end
		
		local t = {}
		for i = 1, math.ceil(#self.gameicon/16) do
			t[i] = {}
			
			for x = 1, 16 do
				local nx = x + 16*(i-1)
				t[i][x] = {}
				
				for y = 1, 16 do
					if self.gameicon[nx] and self.gameicon[nx][y] then
						t[i][x][y] = self.gameicon[nx][y]
					else
						t[i][x][y] = 0
					end
				end
			end
			
		end
		self.gameicon = t
		
		self.animatedicon = {0, 2/#self.gameicon, 1, #self.gameicon}
	end
	
	minigame.loadSettings()
	self.update()
	
	self.arrowTimer = 0
	self.scrollTimer = {0, 4}
	
	self.scrollTimer[1] = self.scrollTimer[2]*0.75
	
	self.levelShake = 0
	self.selection = 1
end

function menu_update(dt)
	self.arrowTimer = self.arrowTimer + dt
	if self.arrowTimer >= 1 then
		self.arrowTimer = self.arrowTimer - 1
	end
	
	self.scrollTimer[1] = self.scrollTimer[1] + dt
	if self.scrollTimer[1] >= self.scrollTimer[2] then
		self.scrollTimer[1] = self.scrollTimer[1] - self.scrollTimer[2]
	end
	
	self.animatedicon[1] = self.animatedicon[1] + dt
	if self.animatedicon[1] >= self.animatedicon[2] then
		self.animatedicon[1] = self.animatedicon[1] - self.animatedicon[2]
		
		self.animatedicon[3] = self.animatedicon[3] + 1
		if self.animatedicon[3] > self.animatedicon[4] then
			self.animatedicon[3] = 1
		end
	end
	
	if self.levelShake > 0 then
		self.levelShake = math.max(0, self.levelShake-dt)
	end
end

function menu_draw()
	local off = 4
	local dis = 8
	local x, y = off, off
	local w, h = minigame.printSize(" ", off)
	h = h/2 - off
	y = y+off*2
	w = w/2 - off
	x = x+w
	x = x - dis*math.sin(math.pi*self.arrowTimer)
	x = math.floor(x)
	if self.selection == 1 then
		minigame.polygon("fill", x+w, y, x+w, y+2*w, x, y+w)
	end
	
	x = off*2+minigame.printWidth(" " .. self.games[currentgame], 4)
	local excess = 0
	
	local screenW = minigame.screens.main.on:getWidth() / minigame.screens.main.scale
	local screenH = minigame.screens.main.on:getHeight() / minigame.screens.main.scale
	
	if x+off*2+dis > screenW then
		excess = (off*2+dis)*2 + minigame.printWidth(" " .. self.games[currentgame], 4) - screenW
		
		local xx = off*2+dis + minigame.printWidth(" ", 4)
		minigame.setScissor(minigame.screens.main.x + (minigame.printWidth(" ", 4) - off/2) * minigame.screens.main.scale, 0, (screenW-xx-dis) * minigame.screens.main.scale, screenH)
		
		local f = math.sin(self.scrollTimer[1]/self.scrollTimer[2]*math.pi*2)
		
		f = (f+1)/2
		minigame.translate(-excess*f, 0)
	end
	minigame.print(" " .. self.games[currentgame], off, off, 4)
	if excess > 0 then
		minigame.setScissor()
		minigame.origin()
		x = x-excess
	end
	x = math.floor(x+dis*math.sin(math.pi*self.arrowTimer))
	
	minigame.setColor(.4)
	minigame.print("    " .. self.gameauthor, off*2+dis, off+minigame.printHeight(" ", 4))
	minigame.setColor(1)
	minigame.print("By: " .. self.gameauthor, off*2+dis-1, off+minigame.printHeight(" ", 4))
	
	if self.selection == 1 then
		minigame.polygon("fill", x, y, x, y+2*w, x+w, y+w)
	end
	
	local startx, starty, rec, dist = off+16, off*2+minigame.printHeight(" ", 4)+16, 4, 1
	if self.gameicon then
		for x = 0, 17 do
			for y = 0, 17 do
				minigame.setColor(.2)
				if x ~= 0 and y ~= 0 and x ~= 17 and y ~= 17 then
					minigame.setColor(self.gameicon[ self.animatedicon[3] ][x][y])
				end
				minigame.rectangle("fill", startx+(x-1)*(rec+dist), starty+(y-1)*(rec+dist), rec, rec)
			end
		end
	end
	
	starty = starty + minigame.printHeight(" ")
	
	local sel = self.selection
	minigame.setColor(.4)
	for a = 0, 1 do
		minigame.print("Speed:" .. (sel == 2 and "  " or "") .. minigame.getSpeed(), startx+18*(rec+dist)+a, starty)
		minigame.print((sel == 4 and "[" or "") .. "Settings" .. (sel == 4 and "]" or ""), startx+18*(rec+dist)+a, starty+minigame.printHeight(" ")*4)
		minigame.setColor(1)
	end
	for a = 0, 1 do
		minigame.setColor(.4)
		if a == 1 then
			minigame.setColor(1)
		end
		minigame.print("Level:" .. (sel == 3 and "  " or ""), startx+18*(rec+dist)+a, starty+minigame.printHeight(" ")*2)
		
		local x = minigame.printWidth("Level:" .. (sel == 3 and "  " or ""))
		if self.levelShake > 0 then
			x = x+math.sin(self.levelShake*math.pi*16)
		end
		if a == 1 then --Highlight when levels aren't available
			minigame.setColor(.4)
			local arc = _G[currentgame .. "_data"].arcadeMode or false
			if minigame.getLevel() <= _G[currentgame .. "_currentLevel"] or arc then
				minigame.setColor(1)
			end
			minigame.print(minigame.getLevel(), startx+18*(rec+dist)+a+x, starty+minigame.printHeight(" ")*2)
		else
			minigame.print(minigame.getLevel(), startx+18*(rec+dist)+a+x, starty+minigame.printHeight(" ")*2)
		end
	end
	if sel == 4 then --reinforce selection bars
		minigame.setColor(.4)
		minigame.print("[", startx+18*(rec+dist)-1, starty+minigame.printHeight(" ")*4)
		minigame.setColor(1)
		minigame.print("[", startx+18*(rec+dist), starty+minigame.printHeight(" ")*4)
		minigame.print("]", startx+18*(rec+dist)+2+minigame.printWidth("[Settings"), starty+minigame.printHeight(" ")*4)
	end
	
	if sel == 2 or sel == 3 then
		minigame.setColor(1)
		local t = {"Speed: ", "Level: ", " " .. minigame.getSpeed(), " " .. minigame.getLevel()}
		local x = startx+18*(rec+dist)+minigame.printWidth(t[sel-1])
		local y = starty-1+(sel-2)*minigame.printHeight(" ")*2
		local w = math.floor(minigame.printWidth(" ")*0.75)
		local h = 2*w
		local dis = 6
		x = x - dis*math.sin(math.pi*self.arrowTimer)
		x = math.floor(x)
		
		minigame.polygon("fill", x, y+h/2, x+w, y+h, x+w, y)
		
		x = startx+18*(rec+dist)+minigame.printWidth(t[sel-1] .. t[sel+1])+minigame.printWidth(" ")/2
		x = x + dis*math.sin(math.pi*self.arrowTimer)
		x = math.floor(x)
		
		minigame.polygon("fill", x+w, y+h/2, x, y+h, x, y)
	end
end

function menu_buttonPressed(button)
	if button == "right" or button == "left" then
		self.levelShake = 0
		if self.selection == 1 then --changing games
			self.scrollTimer[1] = self.scrollTimer[2]*0.75
			if button == "right" then
				self.selectedgame = self.selectedgame + 1
			else
				self.selectedgame = self.selectedgame - 1
			end
			while self.selectedgame < 1 do
				self.selectedgame = self.selectedgame + #games
			end
			
			while self.selectedgame > #games do
				self.selectedgame = self.selectedgame - #games
			end
			
			currentgame = games[self.selectedgame]
			self.update()
			print("Selection = " .. self.selectedgame)
			print("currentgame = " .. currentgame)
		elseif self.selection == 2 then
			if button == "right" then
				minigame.addSpeed(.2)
			else
				minigame.removeSpeed(.2)
			end
		elseif self.selection == 3 then
			if button == "right" then
				gamelevel = math.min(_G[currentgame .. "_maxLevel"], gamelevel+1)
			else
				gamelevel = math.max(1, gamelevel-1)
			end
		end
	end
	
	if button == "up" or button == "down" then
		self.levelShake = 0
		if button == "up" then
			self.selection = self.selection - 1
			if self.selection < 1 then
				self.selection = 4
			end
		else
			self.selection = self.selection + 1
			if self.selection > 4 then
				self.selection = 1
			end
		end
	end
	
	if button == "settings" then
		settings_load(self.selection)
	end
	
	if button == "big" then
		if self.selection == 1 then
			local arc = _G[currentgame .. "_data"].arcadeMode or false
			if minigame.getLevel() <= _G[currentgame .. "_currentLevel"] or arc then
				gamepaused = false
				state = "game"
				leftMenu = "big"
				if _G[currentgame .. "_load"] then
					_G[currentgame .. "_load"](gamelevel)
				end
			else
				self.levelShake = 1
			end
		elseif self.selection == 4 then
			settings_load(self.selection)
		end
	end
end

function menu_buttonReleased(button)
	
end

function alphabetic_order(a, b)
	local g = "abcdefghijklmnopqrstuvwxyz"
	local alphabet = {}
	for i = 1, string.len(g) do table.insert(alphabet, string.sub(g, i, i)) end
	
	local a, b = string.lower(a), string.lower(b)
	for j = 1, math.min(string.len(a), string.len(b)) do
		local aa, bb = string.sub(a, j, j), string.sub(b, j, j)
		if aa ~= bb then
			if table.contains(alphabet, aa) and table.contains(alphabet, bb) then
				if table.contains(alphabet, aa) < table.contains(alphabet, bb) then
					return true
				elseif table.contains(alphabet, aa) > table.contains(alphabet, bb) then
					return false
				end
			elseif table.contains(alphabet, aa) then
				return false
			else
				return true
			end
		elseif j == math.min(string.len(a), string.len(b)) then --virtually similar, return the smallest as first
			return (string.len(a) < string.len(b) and true) or false
		end
	end
end
